"""
队列管理器测试
"""
import pytest
import asyncio
from pi_llm_server.queue_manager import (
    QueueManager,
    ServiceQueue,
    ServiceQueueConfig,
    init_queue_manager,
)


class TestServiceQueue:
    """服务队列测试"""

    @pytest.mark.asyncio
    async def test_queue_process(self):
        """测试队列处理"""
        config = ServiceQueueConfig(max_concurrent=2, max_size=10, timeout_seconds=5)
        queue = ServiceQueue(config)

        async def dummy_task(x):
            await asyncio.sleep(0.1)
            return x * 2

        result = await queue.process_with_queue(dummy_task, 5)
        assert result == 10

    @pytest.mark.asyncio
    async def test_queue_timeout(self):
        """测试队列超时"""
        config = ServiceQueueConfig(max_concurrent=1, max_size=10, timeout_seconds=1)
        queue = ServiceQueue(config)

        async def slow_task():
            await asyncio.sleep(5)
            return "done"

        with pytest.raises(asyncio.TimeoutError):
            await queue.process_with_queue(slow_task)

    @pytest.mark.asyncio
    async def test_queue_status(self):
        """测试队列状态"""
        config = ServiceQueueConfig(max_concurrent=2, max_size=10, timeout_seconds=5)
        queue = ServiceQueue(config)

        status = queue.get_status()
        assert "pending" in status
        assert "processing" in status
        assert "max_concurrent" in status
        assert "max_size" in status


class TestQueueManager:
    """队列管理器测试"""

    def test_add_queue(self):
        """测试添加队列"""
        qm = QueueManager()
        config = ServiceQueueConfig(max_concurrent=2, max_size=10)
        qm.add_queue("test_service", config)

        queue = qm.get_queue("test_service")
        assert queue is not None
        assert queue.config.max_concurrent == 2

    def test_get_nonexistent_queue(self):
        """测试获取不存在的队列"""
        qm = QueueManager()
        queue = qm.get_queue("nonexistent")
        assert queue is None

    @pytest.mark.asyncio
    async def test_process_request(self):
        """测试处理请求"""
        qm = QueueManager()
        config = ServiceQueueConfig(max_concurrent=2, max_size=10, timeout_seconds=5)
        qm.add_queue("test", config)

        async def dummy_task(x):
            return x + 1

        result = await qm.process_request("test", dummy_task, 10)
        assert result == 11

    def test_queue_status(self):
        """测试队列状态"""
        qm = QueueManager()
        qm.add_queue("test1", ServiceQueueConfig())
        qm.add_queue("test2", ServiceQueueConfig())

        status = qm.get_status()
        assert "test1" in status
        assert "test2" in status


class TestInitQueueManager:
    """初始化队列管理器测试"""

    def test_init_queue_manager(self):
        """测试初始化队列管理器"""
        configs = {
            "embedding": ServiceQueueConfig(max_concurrent=4),
            "asr": ServiceQueueConfig(max_concurrent=1),
        }
        qm = init_queue_manager(configs)

        assert qm.get_queue("embedding") is not None
        assert qm.get_queue("asr") is not None
        assert qm.get_queue("embedding").config.max_concurrent == 4
        assert qm.get_queue("asr").config.max_concurrent == 1



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
MinerU API 客户端
将 PDF 文件解析为 Markdown 和图片，并打包成 zip 文件

使用方法:
    python mineru_client.py <input_pdf> <output_zip>
    python mineru_client.py "CX560XAB-2W-BT 产品规格书.pdf" "output.zip"
"""

import os
import sys
import subprocess
import requests
import tempfile
import shutil
import logging
from pathlib import Path
from datetime import datetime

# 配置日志
def setup_logging(service_name: str):
    """配置日志，输出到控制台和文件"""
    logs_dir = os.path.expanduser("~/.cache/pi-llm-server/logs")
    os.makedirs(logs_dir, exist_ok=True)

    log_file = os.path.join(logs_dir, f"{service_name}.log")

    # 创建 logger
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    # 清除已有的 handlers
    logger.handlers = []

    # 创建 formatter
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    # 控制台 handler
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.INFO)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)

    # 文件 handler
    file_handler = logging.FileHandler(log_file, encoding='utf-8')
    file_handler.setLevel(logging.INFO)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    return logger

logger = setup_logging("mineru_client")

# API 服务地址
API_BASE_URL = os.getenv("MINERU_API_URL", "http://127.0.0.1:8094")
API_ENDPOINT = f"{API_BASE_URL}/file_parse"


def check_file_exists(file_path: str) -> tuple:
    """使用 subprocess 检查文件是否存在，避免符号链接编码问题"""
    dir_path = os.path.dirname(file_path)
    filename = os.path.basename(file_path)

    result = subprocess.run(['ls', '-1', dir_path], capture_output=True, text=True)
    files_in_dir = result.stdout.strip().split('\n')

    # 精确匹配
    if filename in files_in_dir:
        return True, os.path.join(dir_path, filename)

    # 模糊匹配：移除空格后比较
    filename_no_space = filename.replace(' ', '').replace('-', '-')
    for f in files_in_dir:
        if f.replace(' ', '') == filename_no_space:
            logger.warning(f"提示：找到相似文件 '{f}' (输入文件名中的空格已被移除)")
            return True, os.path.join(dir_path, f)

    return False, None


def call_mineru_api(input_pdf: str, output_zip: str, backend: str = "pipeline"):
    """
    调用 MinerU API 解析 PDF 文件

    Args:
        input_pdf: 输入的 PDF 文件路径
        output_zip: 输出的 zip 文件路径
        backend: 使用的后端，可选：pipeline, hybrid-auto-engine, vlm-auto-engine 等
    """
    # 检查文件是否存在（使用 subprocess 避免编码问题）
    exists, actual_path = check_file_exists(input_pdf)
    if not exists:
        logger.error(f"文件不存在：{input_pdf}")
        sys.exit(1)

    filename = os.path.basename(actual_path)

    logger.info(f"正在处理文件：{filename}")
    logger.info(f"API 地址：{API_ENDPOINT}")
    logger.info(f"后端：{backend}")

    # 准备请求参数
    files = {
        "files": (filename, open(actual_path, "rb"), "application/pdf")
    }

    data = {
        # 核心参数
        "backend": backend,
        "parse_method": "auto",  # auto/txt/ocr
        "lang_list": "ch",  # 语言：ch/en/korean/japan 等

        # 功能开关
        "formula_enable": "true",   # 启用公式解析
        "table_enable": "true",     # 启用表格解析

        # 返回内容控制
        "return_md": "true",           # 返回 markdown
        "return_images": "true",       # 返回提取的图片
        "return_middle_json": "false", # 返回中间 JSON（可选）
        "return_model_output": "false", # 返回模型输出（可选）
        "return_content_list": "false", # 返回内容列表（可选）

        # 关键：以 zip 格式返回
        "response_format_zip": "true",

        # 页码范围
        "start_page_id": "0",
        "end_page_id": "99999",
    }

    try:
        logger.info("发送请求...")
        logger.info("提示：PDF 解析可能需要几分钟，请耐心等待")
        logger.info("超时时间：1800 秒 (30 分钟)")
        response = requests.post(API_ENDPOINT, files=files, data=data, timeout=1800)

        if response.status_code == 200:
            # 保存 zip 文件
            with open(output_zip, "wb") as f:
                f.write(response.content)
            logger.info(f"成功！输出文件：{output_zip}")

            # 显示 zip 文件内容
            logger.info("ZIP 文件内容:")
            import zipfile
            with zipfile.ZipFile(output_zip, 'r') as zf:
                for name in zf.namelist()[:20]:  # 只显示前 20 个
                    logger.info(f"  {name}")
                if len(zf.namelist()) > 20:
                    logger.info(f"  ... 还有 {len(zf.namelist()) - 20} 个文件")

            logger.info(f"文件大小：{os.path.getsize(output_zip) / 1024 / 1024:.2f} MB")
            return True
        else:
            logger.error(f"请求失败，状态码：{response.status_code}")
            logger.error(f"响应内容：{response.text[:500]}")
            return False

    except requests.exceptions.ConnectionError:
        logger.error(f"无法连接到 API 服务 ({API_BASE_URL})")
        logger.error("请确保已启动 MinerU API 服务：./mineru_server.sh start")
        return False
    except requests.exceptions.Timeout:
        logger.error("请求超时，PDF 文件可能过大或处理时间过长")
        return False
    except Exception as e:
        logger.error(f"错误：{e}")
        return False
    finally:
        if 'files' in locals() and files['files'][1]:
            files['files'][1].close()


def main():
    if len(sys.argv) < 2:
        logger.info(__doc__)
        sys.exit(1)

    input_pdf = sys.argv[1]

    # 自动生成输出文件名
    if len(sys.argv) >= 3:
        output_zip = sys.argv[2]
    else:
        base_name = Path(input_pdf).stem
        output_zip = f"{base_name}_output.zip"

    # 可选：从命令行指定 backend
    backend = sys.argv[3] if len(sys.argv) >= 4 else "pipeline"

    success = call_mineru_api(input_pdf, output_zip, backend)
    sys.exit(0 if success else 1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
ASR 语音识别服务模块

提供语音转文字服务，包括:
- /v1/audio/transcriptions: 语音识别 (form-data 方式)
- /v1/chat/completions: 语音识别 (audio_url 方式)
"""

import httpx
import logging
from typing import Optional, List, Union
from fastapi import APIRouter, HTTPException, UploadFile, File, Form
from fastapi.responses import JSONResponse
from pydantic import BaseModel, Field

from ..config import ServiceConfig

logger = logging.getLogger(__name__)


# ============ 请求/响应模型 ============

class ChatCompletionMessage(BaseModel):
    """Chat 消息"""
    role: str
    content: Union[str, List[dict]]


class ChatCompletionRequest(BaseModel):
    """Chat Completion 请求"""
    model: str
    messages: List[ChatCompletionMessage]
    max_tokens: Optional[int] = Field(default=512, ge=1, le=4096)


class ChatCompletionResponse(BaseModel):
    """Chat Completion 响应"""
    id: str
    object: str
    created: int
    model: str
    choices: List[dict]
    usage: dict


class TranscriptionResponse(BaseModel):
    """语音转写响应"""
    text: str


# ============ 服务类 ============

class ASRService:
    """ASR 服务 (Client + Router)"""

    def __init__(self, config: ServiceConfig):
        """
        初始化 ASR 服务

        Args:
            config: 服务配置
        """
        self.config = config
        self.base_url = config.base_url
        self.timeout = config.timeout_seconds
        self.max_retries = config.max_retries

        # HTTP 客户端 (普通请求)
        self.client = httpx.AsyncClient(
            base_url=self.base_url,
            timeout=httpx.Timeout(self.timeout),
            limits=httpx.Limits(max_keepalive_connections=10, max_connections=20)
        )

        # HTTP 客户端 (长超时，用于长音频)
        self.client_long_timeout = httpx.AsyncClient(
            base_url=self.base_url,
            timeout=httpx.Timeout(max(self.timeout, 600)),
            limits=httpx.Limits(max_keepalive_connections=5, max_connections=10)
        )

        # Router
        self.router = APIRouter(tags=["asr"])
        self._register_routes()

        # 服务信息
        self.models = config.models if hasattr(config, 'models') else []

    async def transcribe_audio(
        self,
        file: UploadFile,
        model: Optional[str] = None,
    ) -> str:
        """
        语音识别 - form-data 方式

        Args:
            file: 音频文件
            model: 模型 ID (可选)

        Returns:
            str: 转写文本

        Raises:
            HTTPException: 请求失败时抛出
        """
        # 读取文件内容
        file_content = await file.read()
        file_filename = file.filename or "audio.wav"

        # 准备请求
        files = {"file": (file_filename, file_content, file.content_type or "audio/wav")}
        data = {}
        if model:
            data["model"] = model

        for attempt in range(self.max_retries):
            try:
                # 使用长超时客户端
                response = await self.client_long_timeout.post(
                    "/v1/audio/transcriptions",
                    files=files,
                    data=data,
                )
                response.raise_for_status()
                result = response.json()
                return result.get("text", "")
            except httpx.TimeoutException as e:
                logger.warning(f"ASR 转录请求超时 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=504, detail="ASR 服务响应超时")
            except httpx.HTTPError as e:
                logger.error(f"ASR 转录请求失败 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=502, detail=f"ASR 服务错误：{str(e)}")

        return ""

    async def chat_completion(
        self,
        messages: List[dict],
        model: str,
        max_tokens: int = 512,
    ) -> dict:
        """
        语音识别 - chat/completions 方式 (audio_url)

        Args:
            messages: 消息列表
            model: 模型 ID
            max_tokens: 最大生成长度

        Returns:
            dict: chat completion 响应

        Raises:
            HTTPException: 请求失败时抛出
        """
        request_data = {
            "model": model,
            "messages": messages,
            "max_tokens": max_tokens,
        }

        for attempt in range(self.max_retries):
            try:
                response = await self.client.post(
                    "/v1/chat/completions",
                    json=request_data,
                )
                response.raise_for_status()
                return response.json()
            except httpx.TimeoutException as e:
                logger.warning(f"ASR chat 请求超时 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=504, detail="ASR 服务响应超时")
            except httpx.HTTPError as e:
                logger.error(f"ASR chat 请求失败 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=502, detail=f"ASR 服务错误：{str(e)}")

        return None

    async def health_check(self) -> dict:
        """
        健康检查

        Returns:
            dict: 健康状态
        """
        try:
            response = await self.client.get("/health", timeout=5.0)
            if response.status_code == 200:
                return {"status": "healthy", "latency_ms": response.elapsed.total_seconds() * 1000}
            else:
                return {"status": "unhealthy", "status_code": response.status_code}
        except Exception as e:
            return {"status": "unhealthy", "error": str(e)}

    async def get_models(self) -> List[dict]:
        """
        获取可用模型列表

        Returns:
            List[dict]: 模型列表
        """
        try:
            response = await self.client.get("/v1/models", timeout=5.0)
            if response.status_code == 200:
                data = response.json()
                # 添加服务信息
                models = data.get("data", [])
                for model in models:
                    model["service"] = "asr"
                return models
            return []
        except Exception as e:
            logger.warning(f"获取模型列表失败：{e}")
            return []

    def _register_routes(self):
        """注册 FastAPI 路由"""

        @self.router.post("/v1/audio/transcriptions", tags=["asr"])
        async def transcriptions_endpoint(
            file: UploadFile = File(..., description="音频文件 (mp3/wav/ogg 等)"),
            model: str = Form(default="Qwen/Qwen3-ASR-1.7B", description="模型 ID"),
        ):
            """语音识别 - form-data 方式"""
            text = await self.transcribe_audio(file, model)
            return {"text": text}

        @self.router.post("/v1/chat/completions", tags=["asr"])
        async def chat_completions_endpoint(request: ChatCompletionRequest):
            """语音识别 - chat/completions 方式 (audio_url)"""
            return await self.chat_completion(
                messages=[m.dict() for m in request.messages],
                model=request.model,
                max_tokens=request.max_tokens,
            )

    async def close(self):
        """关闭 HTTP 客户端"""
        await self.client.aclose()
        await self.client_long_timeout.aclose()


# ============ 全局实例 ============

asr_service: Optional[ASRService] = None


def init_asr_service(config: ServiceConfig) -> ASRService:
    """
    初始化 ASR 服务

    Args:
        config: 服务配置

    Returns:
        ASRService: 服务实例
    """
    global asr_service
    asr_service = ASRService(config)
    logger.info(f"ASR 服务初始化完成：{config.base_url}")
    return asr_service


def get_asr_service() -> ASRService:
    """获取 ASR 服务实例"""
    if not asr_service:
        raise RuntimeError("ASR 服务未初始化")
    return asr_service



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
MinerU PDF 解析服务模块

提供 PDF 文件解析服务，包括:
- /v1/ocr/parser: PDF/OCR 解析 (前端网关路由)
- /file_parse: PDF/OCR 解析 (后端 MinerU API 实际路径)
"""

import httpx
import logging
from typing import Optional, List, Union
from fastapi import APIRouter, HTTPException, UploadFile, File, Form
from fastapi.responses import StreamingResponse, JSONResponse
from pydantic import BaseModel, Field

from ..config import ServiceConfig

logger = logging.getLogger(__name__)


# ============ 请求/响应模型 ============

class MinerURequest(BaseModel):
    """MinerU 解析请求 (JSON 方式)"""
    backend: Optional[str] = Field(default="pipeline", description="解析后端")
    parse_method: Optional[str] = Field(default="auto", description="解析方法")
    lang_list: Optional[str] = Field(default="ch", description="语言")
    formula_enable: Optional[bool] = Field(default=True, description="公式解析")
    table_enable: Optional[bool] = Field(default=True, description="表格解析")
    return_md: Optional[bool] = Field(default=True, description="返回 markdown")
    return_images: Optional[bool] = Field(default=True, description="返回图片")
    return_middle_json: Optional[bool] = Field(default=False, description="返回中间 JSON")
    start_page_id: Optional[int] = Field(default=0, description="起始页")
    end_page_id: Optional[int] = Field(default=99999, description="结束页")


# ============ 服务类 ============

class MinerUService:
    """MinerU 服务 (Client + Router)"""

    def __init__(self, config: ServiceConfig):
        """
        初始化 MinerU 服务

        Args:
            config: 服务配置
        """
        self.config = config
        self.base_url = config.base_url
        self.timeout = config.timeout_seconds
        self.max_retries = config.max_retries

        # HTTP 客户端
        self.client = httpx.AsyncClient(
            base_url=self.base_url,
            timeout=httpx.Timeout(self.timeout),
            limits=httpx.Limits(max_keepalive_connections=5, max_connections=10)
        )

        # Router
        self.router = APIRouter(tags=["mineru"])
        self._register_routes()

        # 服务配置
        self.mineru_config = config.config if hasattr(config, 'config') else {}

    async def parse_pdf(
        self,
        file: UploadFile,
        backend: Optional[str] = "pipeline",
        parse_method: Optional[str] = "auto",
        lang_list: Optional[str] = "ch",
        formula_enable: Optional[bool] = True,
        table_enable: Optional[bool] = True,
        return_md: Optional[bool] = True,
        return_images: Optional[bool] = True,
        return_middle_json: Optional[bool] = False,
        start_page_id: Optional[int] = 0,
        end_page_id: Optional[int] = 99999,
    ) -> bytes:
        """
        解析 PDF 文件

        Args:
            file: PDF 文件
            backend: 解析后端 (pipeline/hybrid-auto-engine/vlm-auto-engine)
            parse_method: 解析方法 (auto/txt/ocr)
            lang_list: 语言 (ch/en/korean/japan 等)
            formula_enable: 是否启用公式解析
            table_enable: 是否启用表格解析
            return_md: 是否返回 markdown
            return_images: 是否返回图片
            return_middle_json: 是否返回中间 JSON
            start_page_id: 起始页码
            end_page_id: 结束页码

        Returns:
            bytes: ZIP 文件内容

        Raises:
            HTTPException: 请求失败时抛出
        """
        # 读取文件内容
        file_content = await file.read()
        file_filename = file.filename or "document.pdf"

        # 准备请求参数
        files = {"files": (file_filename, file_content, "application/pdf")}
        data = {
            "backend": backend,
            "parse_method": parse_method,
            "lang_list": lang_list,
            "formula_enable": str(formula_enable).lower(),
            "table_enable": str(table_enable).lower(),
            "return_md": str(return_md).lower(),
            "return_images": str(return_images).lower(),
            "return_middle_json": str(return_middle_json).lower(),
            "start_page_id": str(start_page_id),
            "end_page_id": str(end_page_id),
            "response_format_zip": "true",  # 关键：以 zip 格式返回
        }

        for attempt in range(self.max_retries):
            try:
                logger.info(f"发送 PDF 解析请求 (attempt {attempt + 1}/{self.max_retries})...")
                logger.info(f"文件：{file_filename}, 后端：{backend}, 超时：{self.timeout}s")

                response = await self.client.post(
                    "/file_parse",
                    files=files,
                    data=data,
                )
                response.raise_for_status()
                return response.content
            except httpx.TimeoutException as e:
                logger.warning(f"MinerU 解析请求超时 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=504, detail="MinerU 服务响应超时")
            except httpx.HTTPStatusError as e:
                logger.error(f"MinerU 解析请求失败 (attempt {attempt + 1}/{self.max_retries}): {e}")
                logger.error(f"响应内容：{e.response.text[:500] if e.response.text else 'N/A'}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(
                        status_code=e.response.status_code,
                        detail=f"MinerU 服务错误：{str(e)}",
                    )
            except httpx.HTTPError as e:
                logger.error(f"MinerU 解析请求失败 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=502, detail=f"MinerU 服务错误：{str(e)}")

        return None

    async def health_check(self) -> dict:
        """
        健康检查

        Returns:
            dict: 健康状态
        """
        # MinerU 没有 /health 端点，检查 /openapi.json
        try:
            response = await self.client.get("/openapi.json", timeout=5.0)
            if response.status_code == 200:
                return {"status": "healthy", "latency_ms": response.elapsed.total_seconds() * 1000}
            else:
                return {"status": "unknown", "status_code": response.status_code}
        except Exception as e:
            # 连接失败时返回 unhealthy
            return {"status": "unhealthy", "error": str(e)}

    async def get_models(self) -> List[dict]:
        """
        获取可用模型列表

        Returns:
            List[dict]: 模型列表
        """
        # MinerU 返回后端类型作为"模型"
        backends = [
            {"id": "mineru/pipeline", "service": "mineru", "type": "pipeline"},
            {"id": "mineru/hybrid-auto-engine", "service": "mineru", "type": "hybrid"},
            {"id": "mineru/vlm-auto-engine", "service": "mineru", "type": "vlm"},
        ]
        return backends

    def _register_routes(self):
        """注册 FastAPI 路由"""

        @self.router.post("/v1/ocr/parser", tags=["mineru"])
        async def parser_endpoint(
            files: UploadFile = File(..., description="PDF 文件"),
            backend: Optional[str] = Form(default="pipeline", description="解析后端"),
            parse_method: Optional[str] = Form(default="auto", description="解析方法"),
            lang_list: Optional[str] = Form(default="ch", description="语言"),
            formula_enable: Optional[bool] = Form(default=True, description="公式解析"),
            table_enable: Optional[bool] = Form(default=True, description="表格解析"),
            return_md: Optional[bool] = Form(default=True, description="返回 markdown"),
            return_images: Optional[bool] = Form(default=True, description="返回图片"),
        ):
            """解析 PDF 文件为 Markdown 和图片"""
            zip_content = await self.parse_pdf(
                file=files,
                backend=backend,
                parse_method=parse_method,
                lang_list=lang_list,
                formula_enable=formula_enable,
                table_enable=table_enable,
                return_md=return_md,
                return_images=return_images,
            )
            return StreamingResponse(
                iter([zip_content]),
                media_type="application/zip",
                headers={"Content-Disposition": "attachment; filename=result.zip"},
            )

    async def close(self):
        """关闭 HTTP 客户端"""
        await self.client.aclose()


# ============ 全局实例 ============

mineru_service: Optional[MinerUService] = None


def init_mineru_service(config: ServiceConfig) -> MinerUService:
    """
    初始化 MinerU 服务

    Args:
        config: 服务配置

    Returns:
        MinerUService: 服务实例
    """
    global mineru_service
    mineru_service = MinerUService(config)
    logger.info(f"MinerU 服务初始化完成：{config.base_url}")
    return mineru_service


def get_mineru_service() -> MinerUService:
    """获取 MinerU 服务实例"""
    if not mineru_service:
        raise RuntimeError("MinerU 服务未初始化")
    return mineru_service



#!/usr/bin/env python3
"""
vllm Embedding 客户端测试工具
用于测试 vllm 提供的 embedding 服务

使用方法:
1. 测试单个文本:
   python vllm_embedding_client.py embed -t "今天天气很好"

2. 批量测试相似度:
   python vllm_embedding_client.py embed-test

3. 语义搜索:
   python vllm_embedding_client.py embed-search -q "人工智能技术"

4. 查看 API 信息:
   python vllm_embedding_client.py info
"""

import os
import sys
import argparse
import requests
import json
import logging
from datetime import datetime

# 配置日志
def setup_logging(service_name: str):
    """配置日志，输出到控制台和文件"""
    logs_dir = os.path.expanduser("~/.cache/pi-llm-server/logs")
    os.makedirs(logs_dir, exist_ok=True)

    log_file = os.path.join(logs_dir, f"{service_name}.log")

    # 创建 logger
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    # 清除已有的 handlers
    logger.handlers = []

    # 创建 formatter
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    # 控制台 handler
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.INFO)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)

    # 文件 handler
    file_handler = logging.FileHandler(log_file, encoding='utf-8')
    file_handler.setLevel(logging.INFO)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    return logger

logger = setup_logging("embedding_client")

# 默认配置
DEFAULT_BASE_URL = "http://localhost:8091"
SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
DATA_DIR = os.path.join(SCRIPT_DIR, "data")
RESULTS_DIR = os.path.join(SCRIPT_DIR, "results")


# ==================== 工具函数 ====================

def cosine_similarity(vec1: list, vec2: list) -> float:
    """计算两个向量的余弦相似度"""
    import math
    dot_product = sum(a * b for a, b in zip(vec1, vec2))
    norm1 = math.sqrt(sum(a * a for a in vec1))
    norm2 = math.sqrt(sum(b * b for b in vec2))
    if norm1 == 0 or norm2 == 0:
        return 0.0
    return dot_product / (norm1 * norm2)


# ==================== 信息查询功能 ====================

def get_server_info(base_url: str):
    """获取服务器信息和可用模型"""
    logger.info("=" * 60)
    logger.info("vllm Embedding 服务器信息")
    logger.info("=" * 60)

    try:
        # 获取模型列表
        response = requests.get(f"{base_url}/v1/models", timeout=30)
        response.raise_for_status()
        models_data = response.json()
        models = models_data.get('data', [])

        logger.info(f"服务器地址：{base_url}, 可用模型数量：{len(models)}")

        for model in models:
            logger.info(f"  - ID: {model.get('id', 'unknown')}")
            logger.info(f"    对象：{model.get('object', 'unknown')}")

        # 测试健康检查
        logger.info("健康检查：OK")

        return models

    except requests.exceptions.ConnectionError:
        logger.error(f"无法连接到服务器：{base_url}, 请确认服务器已启动")
        return []
    except Exception as e:
        logger.error(f"获取服务器信息失败：{e}")
        return []


# ==================== 单个文本 Embedding ====================

def create_embedding(base_url: str, model: str, text: str, encoding_format: str = "float"):
    """
    使用 Embedding 模型生成文本嵌入向量

    Args:
        base_url: vllm 服务地址
        model: 模型名称
        text: 输入文本
        encoding_format: 编码格式：float | base64 (可选，默认 float)
    """
    logger.info("=" * 60)
    logger.info(f"文本嵌入 - 使用模型：{model}")
    logger.info("=" * 60)
    logger.info(f"输入文本：{text[:100]}{'...' if len(text) > 100 else ''}")
    logger.info(f"编码格式：{encoding_format}")
    logger.info("正在计算嵌入向量...")

    url = f"{base_url}/v1/embeddings"

    payload = {
        "model": model,
        "input": text,
        "encoding_format": encoding_format,
    }

    try:
        response = requests.post(url, json=payload, timeout=60)
        response.raise_for_status()

        result = response.json()
        embedding = result.get('data', [{}])[0].get('embedding', [])

        logger.info("生成成功")
        logger.info(f"向量维度：{len(embedding)}")
        logger.info(f"向量预览 (前 10 个值): {embedding[:10]}")

        # 保存到文件
        output_file = os.path.join(RESULTS_DIR, f"embedding_{datetime.now().strftime('%Y%m%d_%H%M%S')}.txt")
        with open(output_file, 'w', encoding='utf-8') as f:
            f.write(f"输入文本：{text}\n")
            f.write(f"模型：{model}\n")
            f.write(f"向量维度：{len(embedding)}\n")
            f.write(f"完整向量:\n{embedding}")

        logger.info(f"完整向量已保存到：{output_file}")

        return embedding

    except Exception as e:
        logger.error(f"错误：{e}")
        return None


# ==================== 批量相似度测试 ====================

def test_embedding_batch(base_url: str, model: str, test_data: list = None):
    """
    批量测试 Embedding 模型，计算文本相似度

    Args:
        base_url: vllm 服务地址
        model: 模型名称
        test_data: 测试文本列表
    """
    logger.info("=" * 60)
    logger.info(f"Embedding 批量测试 - 使用模型：{model}")
    logger.info("=" * 60)

    # 默认测试数据 (按语义分组)
    if test_data is None:
        test_data = [
            # AI 技术相关
            "人工智能是计算机科学的一个分支",
            "AI 技术正在改变各行各业",
            "机器学习是人工智能的核心技术",
            "深度学习用于图像识别和自然语言处理",
            # 天气相关
            "今天天气真好阳光明媚",
            "晴空万里适合外出游玩",
            "天气预报说明天会下雨",
            # 动物相关
            "猫咪喜欢吃鱼和老鼠",
            "小狗是人类最忠诚的朋友",
            "熊猫是中国的国宝动物",
            # 科技产品
            "智能手机已经成为生活必需品",
            "笔记本电脑用于办公和娱乐",
        ]

    url = f"{base_url}/v1/embeddings"

    logger.info(f"测试文本数量：{len(test_data)}")
    logger.info("正在生成嵌入向量...")

    embeddings = []

    try:
        # 为每个文本生成 embedding
        for i, text in enumerate(test_data):
            payload = {
                "model": model,
                "input": text
            }
            response = requests.post(url, json=payload, timeout=60)
            response.raise_for_status()
            result = response.json()
            embedding = result.get('data', [{}])[0].get('embedding', [])
            embeddings.append({
                'text': text,
                'embedding': embedding
            })
            logger.info(f"  [{i+1}/{len(test_data)}] {text[:30]}... -> 维度：{len(embedding)}")

        # 计算相似度矩阵
        logger.info("相似度矩阵:")
        logger.info("-" * 60)

        n = len(embeddings)
        header = "          "
        for i in range(n):
            header += f"[{i+1}]     "
        logger.info(header)

        for i in range(n):
            row = f"[{i+1}] "
            for j in range(n):
                if i == j:
                    sim = 1.000
                elif i > j:
                    sim = cosine_similarity(embeddings[i]['embedding'], embeddings[j]['embedding'])
                else:
                    sim = cosine_similarity(embeddings[i]['embedding'], embeddings[j]['embedding'])
                row += f"{sim:.4f}  "
            logger.info(row + f" <- {embeddings[i]['text'][:25]}")

        # 保存结果
        output_file = os.path.join(RESULTS_DIR, f"embedding_test_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json")
        results = {
            "model": model,
            "timestamp": datetime.now().isoformat(),
            "texts": [e['text'] for e in embeddings],
            "dimension": len(embeddings[0]['embedding']),
            "similarity_matrix": []
        }

        for i in range(n):
            for j in range(i+1, n):
                sim = cosine_similarity(embeddings[i]['embedding'], embeddings[j]['embedding'])
                results['similarity_matrix'].append({
                    "text1": embeddings[i]['text'],
                    "text2": embeddings[j]['text'],
                    "similarity": round(sim, 4)
                })

        with open(output_file, 'w', encoding='utf-8') as f:
            json.dump(results, f, ensure_ascii=False, indent=2)

        logger.info(f"详细结果已保存到：{output_file}")

        # 显示高相似度对
        logger.info("高相似度文本对 (相似度 > 0.85):")
        logger.info("-" * 60)
        high_sim_count = 0
        for item in results['similarity_matrix']:
            if item['similarity'] > 0.85:
                high_sim_count += 1
                logger.info(f"  {item['similarity']:.4f}: {item['text1'][:20]}... <-> {item['text2'][:20]}...")

        if high_sim_count == 0:
            logger.info("无高相似度文本对")

    except Exception as e:
        logger.error(f"错误：{e}")


# ==================== 语义搜索 ====================

def test_embedding_search(base_url: str, model: str, query: str, documents: list = None):
    """
    使用 Embedding 模型进行语义搜索测试

    Args:
        base_url: vllm 服务地址
        model: 模型名称
        query: 查询文本
        documents: 文档列表
    """
    logger.info("=" * 60)
    logger.info(f"Embedding 语义搜索测试 - 使用模型：{model}")
    logger.info("=" * 60)

    # 默认文档库
    if documents is None:
        documents = [
            "人工智能是计算机科学的一个分支，致力于创建智能系统",
            "机器学习使用算法让计算机从数据中学习",
            "深度学习是机器学习的子集，使用神经网络",
            "自然语言处理让计算机理解和生成人类语言",
            "计算机视觉让计算机能够理解图像和视频",
            "语音识别技术将语音转换为文本",
            "推荐系统根据用户偏好推荐内容",
            "数据挖掘从大量数据中发现模式和知识",
            "机器人技术设计和制造智能机器",
            "算法是解决问题的步骤和规则",
            "大数据技术处理海量信息的存储和分析",
            "云计算提供按需计算资源和服务",
            "物联网连接物理设备和传感器",
            "区块链是分布式账本技术用于安全交易",
            "网络安全保护系统免受攻击",
            "Python 是流行的编程语言用于 AI 开发",
        ]

    if not query:
        logger.error("错误：请提供查询文本")
        return

    url = f"{base_url}/v1/embeddings"

    logger.info(f"查询：{query}")
    logger.info(f"文档库大小：{len(documents)}")
    logger.info("正在计算向量...")

    try:
        # 生成查询向量
        payload = {"model": model, "input": query}
        response = requests.post(url, json=payload, timeout=60)
        response.raise_for_status()
        query_embedding = response.json().get('data', [{}])[0].get('embedding', [])

        # 生成所有文档向量
        doc_embeddings = []
        for doc in documents:
            payload = {"model": model, "input": doc}
            response = requests.post(url, json=payload, timeout=60)
            response.raise_for_status()
            embedding = response.json().get('data', [{}])[0].get('embedding', [])
            doc_embeddings.append({'text': doc, 'embedding': embedding})
            logger.info(f"已处理文档：{doc[:30]}... -> 维度：{len(embedding)}")

        # 计算相似度并排序
        results = []
        for doc in doc_embeddings:
            sim = cosine_similarity(query_embedding, doc['embedding'])
            results.append({'text': doc['text'], 'similarity': sim})

        results.sort(key=lambda x: x['similarity'], reverse=True)

        logger.info("搜索结果 (按相似度排序):")
        logger.info("-" * 60)
        for i, result in enumerate(results):
            score = result['similarity']
            bar = "█" * int(score * 20)
            logger.info(f"  [{i+1}] 相似度：{score:.4f} {bar}")
            logger.info(f"      文档：{result['text']}")

        # 保存结果
        output_file = os.path.join(RESULTS_DIR, f"embedding_search_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json")
        search_results = {
            "model": model,
            "timestamp": datetime.now().isoformat(),
            "query": query,
            "results": [{"rank": i+1, "text": r['text'], "similarity": round(r['similarity'], 4)} for i, r in enumerate(results)]
        }

        with open(output_file, 'w', encoding='utf-8') as f:
            json.dump(search_results, f, ensure_ascii=False, indent=2)

        logger.info(f"详细结果已保存到：{output_file}")

    except Exception as e:
        logger.error(f"错误：{e}")


# ==================== 主函数 ====================

def main():
    parser = argparse.ArgumentParser(
        description="vllm Embedding 客户端测试工具",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
示例用法:
  # 查看服务器信息
  python vllm_embedding_client.py info

  # 测试单个文本
  python vllm_embedding_client.py embed -t "今天天气很好"

  # 批量相似度测试
  python vllm_embedding_client.py embed-test

  # 语义搜索
  python vllm_embedding_client.py embed-search -q "人工智能技术"
        """
    )

    parser.add_argument('--base-url', default=DEFAULT_BASE_URL,
                        help=f'vllm 服务地址 (默认：{DEFAULT_BASE_URL})')

    subparsers = parser.add_subparsers(dest='command', help='命令类型')

    # info 命令
    info_parser = subparsers.add_parser('info', help='查看服务器信息')

    # embed 命令 (单个文本)
    embed_parser = subparsers.add_parser('embed', help='文本嵌入 (单个)')
    embed_parser.add_argument('--model', '-m', default='unsloth/Qwen3-Embedding-0.6B',
                              help='Embedding 模型名称 (默认：unsloth/Qwen3-Embedding-0.6B)')
    embed_parser.add_argument('--text', '-t', required=True, help='输入文本')
    embed_parser.add_argument('--encoding-format', '-e', default='float',
                              help='编码格式：float | base64 (默认：float)')

    # embed-test 命令 (批量测试)
    embed_test_parser = subparsers.add_parser('embed-test', help='Embedding 批量测试 (相似度矩阵)')
    embed_test_parser.add_argument('--model', '-m', default='unsloth/Qwen3-Embedding-0.6B',
                                   help='Embedding 模型名称 (默认：unsloth/Qwen3-Embedding-0.6B)')

    # embed-search 命令 (语义搜索)
    embed_search_parser = subparsers.add_parser('embed-search', help='Embedding 语义搜索')
    embed_search_parser.add_argument('--model', '-m', default='unsloth/Qwen3-Embedding-0.6B',
                                     help='Embedding 模型名称 (默认：unsloth/Qwen3-Embedding-0.6B)')
    embed_search_parser.add_argument('--query', '-q', required=True, help='查询文本')

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
        return

    # 执行对应命令
    if args.command == 'info':
        get_server_info(args.base_url)

    elif args.command == 'embed':
        create_embedding(args.base_url, args.model, args.text, args.encoding_format)

    elif args.command == 'embed-test':
        test_embedding_batch(args.base_url, args.model)

    elif args.command == 'embed-search':
        test_embedding_search(args.base_url, args.model, args.query)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Qwen3-Embedding 服务器
使用 HuggingFace sentence-transformers + FastAPI 运行 embedding 服务
支持 Qwen3-Embedding-0.6B 和 Qwen3-Embedding-4B 模型

依赖安装:
    pip install fastapi uvicorn sentence-transformers torch

使用方法:
    # 使用 Qwen3-Embedding-0.6B (默认 CPU)
    python embedding_server.py

    # 使用 GPU
    python embedding_server.py --device cuda

    # 使用 Qwen3-Embedding-4B (需要 >10GB 显存)
    python embedding_server.py --device cuda --model-path /path/to/Qwen3-Embedding-4B

    # 指定 GPU
    python embedding_server.py --device cuda:1

    # 后台运行
    nohup python embedding_server.py > server.log 2>&1 &
"""

import os
import sys
import argparse
import logging
import time
import base64
import struct
from typing import List, Optional, Union

import torch
from fastapi import FastAPI, HTTPException
from pydantic import BaseModel
import uvicorn

# 导入版本号
from pi_llm_server import __version__

# 尝试导入 sentence-transformers
try:
    from sentence_transformers import SentenceTransformer
except ImportError:
    logger.error("需要安装 sentence-transformers")
    logger.error("请运行：pip install sentence-transformers")
    sys.exit(1)

# 配置日志
def setup_logging(service_name: str):
    """配置日志，输出到控制台和文件"""
    logs_dir = os.path.expanduser("~/.cache/pi-llm-server/logs")
    os.makedirs(logs_dir, exist_ok=True)

    log_file = os.path.join(logs_dir, f"{service_name}.log")

    # 创建 logger
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    # 清除已有的 handlers
    logger.handlers = []

    # 创建 formatter
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    # 控制台 handler
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.INFO)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)

    # 文件 handler
    file_handler = logging.FileHandler(log_file, encoding='utf-8')
    file_handler.setLevel(logging.INFO)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    return logger

logger = setup_logging("embedding")

# FastAPI 应用
app = FastAPI(
    title="Qwen3-Embedding API",
    description="Qwen3-Embedding 模型 embedding 服务",
    version=__version__
)

# 全局变量
model = None
device = None
model_name = None

# 默认配置
DEFAULT_MODEL_PATH = os.path.expanduser("~/.cache/modelscope/hub/models/unsloth/Qwen3-Embedding-0.6B")
DEFAULT_HOST = "0.0.0.0"
DEFAULT_PORT = 8091


# ==================== 请求/响应模型 ====================

class EmbeddingRequest(BaseModel):
    input: Union[str, List[str]]
    model: str = ""
    encoding_format: Optional[str] = "float"  # "float" | "base64"


class EmbeddingData(BaseModel):
    object: str = "embedding"
    embedding: List[float]
    index: int


class EmbeddingUsage(BaseModel):
    prompt_tokens: int
    total_tokens: int


class EmbeddingResponse(BaseModel):
    object: str = "list"
    data: List[EmbeddingData]
    model: str
    usage: EmbeddingUsage


class SimilarityRequest(BaseModel):
    text1: str
    text2: str


# ==================== 模型加载 ====================

def load_model(model_path: str, device_str: str = "cuda"):
    """加载 embedding 模型"""
    global model, device, model_name

    # 从路径提取模型名
    model_name = os.path.basename(os.path.dirname(model_path)) + "/" + os.path.basename(model_path)

    # 判断设备
    use_cuda = device_str.startswith("cuda") and torch.cuda.is_available()

    if device_str.startswith("cuda") and not torch.cuda.is_available():
        logger.warning("未检测到 GPU，自动切换到 CPU")

    if use_cuda:
        gpu_idx = 0
        if ":" in device_str:
            gpu_idx = int(device_str.split(":")[1])
        gpu_name = torch.cuda.get_device_name(gpu_idx)
        gpu_memory = torch.cuda.get_device_properties(gpu_idx).total_memory / 1024**3
        logger.info(f"检测到 GPU: {gpu_name} ({gpu_memory:.1f} GB)")

    logger.info(f"正在加载模型：{model_path}")

    try:
        if use_cuda:
            logger.info(f"尝试使用 {device_str} 加载...")
            model = SentenceTransformer(
                model_path,
                device=device_str,
                trust_remote_code=True
            )
            device = torch.device(device_str)
            logger.info(f"模型加载成功，使用 GPU: {gpu_name}")
        else:
            logger.info("使用 CPU 加载...")
            model = SentenceTransformer(
                model_path,
                device="cpu",
                trust_remote_code=True,
                model_kwargs={"low_cpu_mem_usage": True}
            )
            device = torch.device("cpu")
            logger.info("模型加载成功，使用 CPU")

    except torch.OutOfMemoryError as e:
        if use_cuda:
            logger.warning(f"GPU 显存不足，切换到 CPU: {e}")
            model = SentenceTransformer(
                model_path,
                device="cpu",
                trust_remote_code=True,
                model_kwargs={"low_cpu_mem_usage": True}
            )
            device = torch.device("cpu")
            logger.info("模型已在 CPU 上加载")
        else:
            raise

    # 显示模型信息
    total_params = sum(p.numel() for p in model.parameters())
    embedding_dim = model.get_sentence_embedding_dimension()
    logger.info(f"模型参数：{total_params / 1e6:.1f}M, 向量维度：{embedding_dim}")


# ==================== API 端点 ====================

@app.get("/health")
async def health_check():
    """健康检查"""
    return {"status": "healthy", "device": str(device) if device else "not_loaded"}


@app.get("/v1/models")
async def list_models():
    """列出可用模型"""
    return {
        "object": "list",
        "data": [
            {
                "id": model_name,
                "object": "model",
                "owned_by": "local"
            }
        ]
    }


@app.post("/v1/embeddings")
async def create_embeddings(request: EmbeddingRequest):
    """OpenAI 兼容的 embedding 接口"""
    if model is None:
        raise HTTPException(status_code=503, detail="模型未加载")

    try:
        # 处理输入
        if isinstance(request.input, str):
            input_texts = [request.input]
        else:
            input_texts = request.input

        if not input_texts:
            raise HTTPException(status_code=400, detail="输入文本不能为空")

        start_time = time.time()

        # 生成 embedding
        embeddings = model.encode(
            input_texts,
            convert_to_tensor=False,
            show_progress_bar=False
        )

        elapsed = time.time() - start_time
        logger.info(f"处理 {len(input_texts)} 条文本，耗时 {elapsed:.3f}s")

        # 精确计算 token 数
        tokenizer = model.tokenizer
        total_tokens = sum(len(tokenizer.encode(t)) for t in input_texts)

        # 构建响应 - 支持 float 和 base64 格式
        data = []
        for i, emb in enumerate(embeddings):
            emb_list = emb.tolist() if hasattr(emb, 'tolist') else list(emb)

            if request.encoding_format == "base64":
                # 将 float 数组编码为 base64
                # 使用 float32 打包
                packed = struct.pack(f'{len(emb_list)}f', *emb_list)
                encoded = base64.b64encode(packed).decode('ascii')
                data.append(EmbeddingData(
                    embedding=encoded,
                    index=i
                ))
            else:
                # 默认 float 格式
                data.append(EmbeddingData(
                    embedding=emb_list,
                    index=i
                ))

        return EmbeddingResponse(
            data=data,
            model=model_name,
            usage=EmbeddingUsage(
                prompt_tokens=total_tokens,
                total_tokens=total_tokens
            )
        )

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"生成 embedding 失败：{e}")
        raise HTTPException(status_code=500, detail=str(e))


@app.post("/v1/similarity")
async def calculate_similarity(request: SimilarityRequest):
    """
    计算两个文本的余弦相似度

    响应格式: {"similarity": 0.95}
    """
    if model is None:
        raise HTTPException(status_code=503, detail="模型未加载")

    try:
        emb1 = model.encode(request.text1, convert_to_tensor=True)
        emb2 = model.encode(request.text2, convert_to_tensor=True)

        sim = torch.nn.functional.cosine_similarity(
            emb1.unsqueeze(0), emb2.unsqueeze(0)
        ).item()

        return {"similarity": sim}

    except Exception as e:
        logger.error(f"计算相似度失败：{e}")
        raise HTTPException(status_code=500, detail=str(e))


# ==================== 主函数 ====================

def main():
    parser = argparse.ArgumentParser(
        description="Qwen3-Embedding 服务器 (HuggingFace + FastAPI)",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
示例用法:
  # 基本启动 (默认 CPU)
  python embedding_server.py

  # GPU 模式
  python embedding_server.py --device cuda

  # 指定 GPU
  python embedding_server.py --device cuda:1

  # 自定义端口和模型
  python embedding_server.py --port 8000 --model-path /path/to/model

  # 后台运行
  nohup python embedding_server.py > server.log 2>&1 &
        """
    )

    parser.add_argument(
        "--model-path",
        type=str,
        default=DEFAULT_MODEL_PATH,
        help=f"模型路径 (默认：{DEFAULT_MODEL_PATH})"
    )
    parser.add_argument(
        "--host",
        type=str,
        default=DEFAULT_HOST,
        help=f"监听地址 (默认：{DEFAULT_HOST})"
    )
    parser.add_argument(
        "--port",
        type=int,
        default=DEFAULT_PORT,
        help=f"监听端口 (默认：{DEFAULT_PORT})"
    )
    parser.add_argument(
        "--device",
        type=str,
        default="cpu",
        help="运行设备 (默认：cpu，可选：cpu, cuda, cuda:0, cuda:1 等)"
    )

    args = parser.parse_args()

    # 检查模型路径
    if not os.path.exists(args.model_path):
        logger.error(f"模型路径不存在：{args.model_path}")
        sys.exit(1)

    # 加载模型
    load_model(args.model_path, device_str=args.device)

    # 启动服务
    logger.info(f"启动服务器：http://{args.host}:{args.port}")
    uvicorn.run(
        app,
        host=args.host,
        port=args.port,
        log_level="info"
    )


if __name__ == "__main__":
    main()



"""
配置模块测试
"""
import pytest
from pathlib import Path
from pi_llm_server.config import ConfigManager, Config, ServiceConfig


# 示例配置文件路径
EXAMPLE_CONFIG = Path(__file__).parent.parent / "examples" / "config.example.yaml"


class TestConfigManager:
    """配置管理器测试"""

    def test_config_load(self):
        """测试配置文件加载"""
        config = ConfigManager(str(EXAMPLE_CONFIG))
        assert config.server.port == 8090
        assert config.server.host == "0.0.0.0"
        assert config.server.workers == 4

    def test_auth_config(self):
        """测试认证配置"""
        config = ConfigManager(str(EXAMPLE_CONFIG))
        assert config.auth.enabled is True
        assert len(config.auth.tokens) > 0
        assert "sk-5f8b839908d14561590b70227c72ca86" in config.auth.tokens

    def test_queue_config(self):
        """测试队列配置"""
        config = ConfigManager(str(EXAMPLE_CONFIG))
        assert config.queue.enabled is True
        # 检查默认配置
        assert config.queue.default.max_concurrent == 1
        assert config.queue.default.max_size == 100

    def test_service_queue_config(self):
        """测试服务队列配置"""
        config = ConfigManager(str(EXAMPLE_CONFIG))
        # 检查 embedding 队列配置
        emb_queue = config.get_queue_config("embedding")
        assert emb_queue.max_concurrent == 4
        assert emb_queue.max_size == 200

        # 检查 asr 队列配置
        asr_queue = config.get_queue_config("asr")
        assert asr_queue.max_concurrent == 1
        assert asr_queue.max_size == 50

    def test_service_config(self):
        """测试服务配置"""
        config = ConfigManager(str(EXAMPLE_CONFIG))
        embedding_cfg = config.get_service_config("embedding")
        assert embedding_cfg is not None
        assert embedding_cfg.enabled is True
        assert embedding_cfg.base_url == "http://127.0.0.1:8091"

    def test_health_check_config(self):
        """测试健康检查配置"""
        config = ConfigManager(str(EXAMPLE_CONFIG))
        hc = config.health_check
        assert hc.enabled is True
        assert hc.interval_seconds == 30
        assert hc.timeout_seconds == 10
        assert hc.unhealthy_threshold == 3

    def test_get_enabled_services(self):
        """测试获取启用的服务列表"""
        config = ConfigManager(str(EXAMPLE_CONFIG))
        services = config.get_enabled_services()
        assert "embedding" in services
        assert "asr" in services
        assert "reranker" in services
        assert "mineru" in services

    def test_token_validation(self):
        """测试 Token 验证"""
        config = ConfigManager(str(EXAMPLE_CONFIG))
        # 有效 token
        assert config.validate_token("sk-5f8b839908d14561590b70227c72ca86", "/v1/embeddings")
        # 无效 token
        assert not config.validate_token("invalid-token", "/v1/embeddings")


class TestConfigModels:
    """配置模型测试"""

    def test_server_config(self):
        """测试服务器配置模型"""
        from pi_llm_server.config import ServerConfig
        cfg = ServerConfig()
        assert cfg.host == "0.0.0.0"
        assert cfg.port == 8090

    def test_service_config(self):
        """测试服务配置模型"""
        cfg = ServiceConfig(
            enabled=True,
            base_url="http://localhost:8000"
        )
        assert cfg.enabled is True
        assert cfg.base_url == "http://localhost:8000"
        assert cfg.timeout_seconds == 300
        assert cfg.max_retries == 3

    def test_model_path_resolution(self):
        """测试模型路径解析（~ 展开）"""
        from pi_llm_server.config import ModelConfig
        import os

        # 测试 ~ 路径解析
        cfg = ModelConfig(
            id="test/model",
            path="~/.cache/modelscope/hub/models/test"
        )
        expected_path = os.path.expanduser("~/.cache/modelscope/hub/models/test")
        assert cfg.path == expected_path

    def test_model_path_with_env(self):
        """测试模型路径环境变量解析"""
        from pi_llm_server.config import ModelConfig
        import os

        # 设置环境变量
        os.environ["TEST_MODEL_PATH"] = "/tmp/test_model"

        # 测试环境变量路径解析
        cfg = ModelConfig(
            id="test/model",
            path="$TEST_MODEL_PATH/model"
        )
        assert cfg.path == "/tmp/test_model/model"

    def test_model_config_from_yaml(self):
        """测试从 YAML 配置加载模型路径"""
        config = ConfigManager(str(EXAMPLE_CONFIG))

        # 检查 embedding 模型路径
        embedding_cfg = config.get_service_config("embedding")
        assert embedding_cfg is not None
        assert len(embedding_cfg.models) > 0

        model = embedding_cfg.models[0]
        # 验证路径已经被解析（不包含 ~）
        assert "~" not in model.path
        assert "/.cache/modelscope" in model.path or ".cache/modelscope" in model.path



#!/usr/bin/env python3
"""
Qwen3-Reranker 服务器
使用 HuggingFace Transformers + FastAPI 运行 reranker 服务
支持 GPU/CPU 模式

依赖安装:
    pip install fastapi uvicorn torch transformers

使用方法:
    # 基本启动 (默认 CPU)
    python reranker_server.py

    # 使用 GPU
    python reranker_server.py --device cuda

    # 指定 GPU
    python reranker_server.py --device cuda:1

    # 自定义端口
    python reranker_server.py --port 8000

    # 后台运行
    nohup python reranker_server.py > server.log 2>&1 &
"""

import os
import sys
import argparse
import logging
import time
from typing import List, Optional

import torch
from fastapi import FastAPI, HTTPException
from pydantic import BaseModel
import uvicorn

# 导入版本号
from pi_llm_server import __version__

# 尝试导入 transformers
try:
    from transformers import AutoModelForCausalLM, AutoTokenizer
except ImportError:
    logger.error("需要安装 transformers")
    logger.error("请运行：pip install transformers torch")
    sys.exit(1)

# 配置日志
def setup_logging(service_name: str):
    """配置日志，输出到控制台和文件"""
    logs_dir = os.path.expanduser("~/.cache/pi-llm-server/logs")
    os.makedirs(logs_dir, exist_ok=True)

    log_file = os.path.join(logs_dir, f"{service_name}.log")

    # 创建 logger
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    # 清除已有的 handlers
    logger.handlers = []

    # 创建 formatter
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    # 控制台 handler
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.INFO)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)

    # 文件 handler
    file_handler = logging.FileHandler(log_file, encoding='utf-8')
    file_handler.setLevel(logging.INFO)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    return logger

logger = setup_logging("reranker")

# FastAPI 应用
app = FastAPI(
    title="Qwen3-Reranker API",
    description="Qwen3-Reranker 模型 reranker 服务",
    version=__version__
)

# 全局变量
model = None
tokenizer = None
device = None
model_name = None

# 默认配置
DEFAULT_MODEL_PATH = os.path.expanduser("~/.cache/modelscope/hub/models/Qwen/Qwen3-Reranker-0.6B")
DEFAULT_HOST = "0.0.0.0"
DEFAULT_PORT = 8093


# ==================== 请求/响应模型 ====================

class RerankRequest(BaseModel):
    model: Optional[str] = ""
    query: str
    documents: List[str]
    top_n: Optional[int] = None
    instruction: Optional[str] = None
    encoding_format: Optional[str] = None  # 保留用于未来扩展


class RerankResult(BaseModel):
    index: int
    document: str
    relevance_score: float


class RerankUsage(BaseModel):
    prompt_tokens: int
    total_tokens: int


class RerankResponse(BaseModel):
    model: str
    results: List[RerankResult]
    usage: RerankUsage


# ==================== 模型加载 ====================

def load_model(model_path: str, device_str: str = "cpu"):
    """加载 reranker 模型"""
    global model, tokenizer, device, model_name

    # 从路径提取模型名
    model_name = os.path.basename(os.path.dirname(model_path)) + "/" + os.path.basename(model_path)

    # 判断设备
    use_cuda = device_str.startswith("cuda") and torch.cuda.is_available()

    if device_str.startswith("cuda") and not torch.cuda.is_available():
        logger.warning("未检测到 GPU，自动切换到 CPU")

    if use_cuda:
        gpu_idx = 0
        if ":" in device_str:
            gpu_idx = int(device_str.split(":")[1])
        gpu_name = torch.cuda.get_device_name(gpu_idx)
        gpu_memory = torch.cuda.get_device_properties(gpu_idx).total_memory / 1024**3
        logger.info(f"检测到 GPU: {gpu_name} ({gpu_memory:.1f} GB)")

    logger.info(f"正在加载模型：{model_path}")

    try:
        tokenizer = AutoTokenizer.from_pretrained(
            model_path,
            trust_remote_code=True
        )

        if use_cuda:
            logger.info(f"尝试使用 {device_str} 加载...")
            model = AutoModelForCausalLM.from_pretrained(
                model_path,
                trust_remote_code=True,
                dtype=torch.float16
            )
            device = torch.device(device_str)
            model.to(device)
            logger.info(f"模型加载成功，使用 GPU: {gpu_name}")
        else:
            logger.info("使用 CPU 加载...")
            model = AutoModelForCausalLM.from_pretrained(
                model_path,
                trust_remote_code=True,
                dtype=torch.float32,
                low_cpu_mem_usage=True
            )
            device = torch.device("cpu")
            logger.info("模型加载成功，使用 CPU")

    except torch.OutOfMemoryError as e:
        if use_cuda:
            logger.warning(f"GPU 显存不足，切换到 CPU: {e}")
            model = AutoModelForCausalLM.from_pretrained(
                model_path,
                trust_remote_code=True,
                dtype=torch.float32,
                low_cpu_mem_usage=True
            )
            device = torch.device("cpu")
            logger.info("模型已在 CPU 上加载")
        else:
            raise
    except Exception as e:
        logger.error(f"模型加载失败：{e}")
        sys.exit(1)

    model.eval()

    # 显示模型信息
    total_params = sum(p.numel() for p in model.parameters())
    logger.info(f"模型参数：{total_params / 1e6:.1f}M")


# ==================== Rerank 计算 ====================

# Qwen3-Reranker 的 prompt 模板
RERANK_PREFIX = '<|im_start|>system\nJudge whether the Document is relevant to the Query. Answer only "yes" or "no".<|im_end|>\n<|im_start|>user\n'
RERANK_SUFFIX = '<|im_end|>\n<|im_start|>assistant\n'
RERANK_TOKEN_FALSE = 'no'
RERANK_TOKEN_TRUE = 'yes'


def format_pairs(query: str, documents: List[str]) -> List[str]:
    """格式化 query-document 对"""
    pairs = []
    for doc in documents:
        text = f"{RERANK_PREFIX}<Query>{query}</Query>\n<Document>{doc}</Document>{RERANK_SUFFIX}"
        pairs.append(text)
    return pairs


def compute_rerank_scores(query: str, documents: List[str]) -> List[float]:
    """计算 rerank 分数"""
    pairs = format_pairs(query, documents)

    # 获取 yes/no token id
    token_false_id = tokenizer.convert_tokens_to_ids(RERANK_TOKEN_FALSE)
    token_true_id = tokenizer.convert_tokens_to_ids(RERANK_TOKEN_TRUE)

    scores = []
    for text in pairs:
        inputs = tokenizer(
            text,
            return_tensors="pt",
            padding=True,
            truncation=True,
            max_length=8192
        )
        inputs = {k: v.to(device) for k, v in inputs.items()}

        with torch.no_grad():
            outputs = model(**inputs)
            # CausalLM: 取最后一个 token 的 logits（预测下一个 token）
            logits = outputs.logits[:, -1, :]

            # 获取 yes/no 的 logit
            true_logit = logits[0, token_true_id].item()
            false_logit = logits[0, token_false_id].item()

            # sigmoid 归一化
            import math
            score = 1.0 / (1.0 + math.exp(-(true_logit - false_logit)))
            scores.append(score)

    return scores


# ==================== API 端点 ====================

@app.get("/health")
async def health_check():
    """健康检查"""
    return {"status": "healthy", "device": str(device) if device else "not_loaded"}


@app.get("/v1/models")
async def list_models():
    """列出可用模型"""
    return {
        "object": "list",
        "data": [
            {
                "id": model_name,
                "object": "model",
                "owned_by": "local"
            }
        ]
    }


@app.post("/v1/rerank")
async def rerank(request: RerankRequest):
    """Rerank 接口"""
    if model is None:
        raise HTTPException(status_code=503, detail="模型未加载")

    try:
        if not request.query:
            raise HTTPException(status_code=400, detail="query 不能为空")
        if not request.documents:
            raise HTTPException(status_code=400, detail="documents 不能为空")

        start_time = time.time()
        scores = compute_rerank_scores(request.query, request.documents)
        elapsed = time.time() - start_time

        logger.info(f"Rerank {len(request.documents)} 条文档，耗时 {elapsed:.3f}s")

        # 构建结果
        results = [
            RerankResult(
                index=i,
                document=doc,
                relevance_score=score
            )
            for i, (doc, score) in enumerate(zip(request.documents, scores))
        ]

        # 按分数降序排列
        results.sort(key=lambda x: x.relevance_score, reverse=True)

        # top_n 截断
        if request.top_n is not None and request.top_n > 0:
            results = results[:request.top_n]

        # 计算 token 数
        total_tokens = sum(
            len(tokenizer.encode(f"{request.query} {doc}"))
            for doc in request.documents
        )

        return RerankResponse(
            model=model_name,
            results=results,
            usage=RerankUsage(
                prompt_tokens=total_tokens,
                total_tokens=total_tokens
            )
        )

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Rerank 失败：{e}")
        raise HTTPException(status_code=500, detail=str(e))


# ==================== 主函数 ====================

def main():
    parser = argparse.ArgumentParser(
        description="Qwen3-Reranker 服务器 (HuggingFace + FastAPI)",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
示例用法:
  # 基本启动 (默认 CPU)
  python reranker_server.py

  # GPU 模式
  python reranker_server.py --device cuda

  # 指定 GPU
  python reranker_server.py --device cuda:1

  # 自定义端口和模型
  python reranker_server.py --port 8000 --model-path /path/to/model

  # 后台运行
  nohup python reranker_server.py > server.log 2>&1 &
        """
    )

    parser.add_argument(
        "--model-path",
        type=str,
        default=DEFAULT_MODEL_PATH,
        help=f"模型路径 (默认：{DEFAULT_MODEL_PATH})"
    )
    parser.add_argument(
        "--host",
        type=str,
        default=DEFAULT_HOST,
        help=f"监听地址 (默认：{DEFAULT_HOST})"
    )
    parser.add_argument(
        "--port",
        type=int,
        default=DEFAULT_PORT,
        help=f"监听端口 (默认：{DEFAULT_PORT})"
    )
    parser.add_argument(
        "--device",
        type=str,
        default="cpu",
        help="运行设备 (默认：cpu，可选：cpu, cuda, cuda:0, cuda:1 等)"
    )

    args = parser.parse_args()

    # 检查模型路径
    if not os.path.exists(args.model_path):
        logger.error(f"模型路径不存在：{args.model_path}")
        sys.exit(1)

    # 加载模型
    load_model(args.model_path, device_str=args.device)

    # 启动服务
    logger.info(f"启动服务器：http://{args.host}:{args.port}")
    uvicorn.run(
        app,
        host=args.host,
        port=args.port,
        log_level="info"
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
队列管理模块

实现差异化请求队列管理，支持按服务类型配置不同的并发策略。
- Embedding/Reranker: CPU 多核并行
- ASR/MinerU: GPU 顺序处理
"""

import asyncio
from typing import Dict, Callable, Any, Optional
from dataclasses import dataclass, field
from datetime import datetime
import logging

logger = logging.getLogger(__name__)


@dataclass
class ServiceQueueConfig:
    """服务队列配置"""
    max_concurrent: int = 1
    max_size: int = 100
    timeout_seconds: int = 300


@dataclass
class QueueStatus:
    """队列状态"""
    pending: int = 0
    processing: int = 0
    total_processed: int = 0
    total_rejected: int = 0
    last_processed_time: Optional[datetime] = None


class ServiceQueue:
    """单个服务的队列"""

    def __init__(self, config: ServiceQueueConfig):
        """
        初始化服务队列

        Args:
            config: 队列配置
        """
        self.config = config
        self.semaphore = asyncio.Semaphore(config.max_concurrent)
        self.queue = asyncio.Queue(maxsize=config.max_size)
        self.status = QueueStatus()
        self._lock = asyncio.Lock()

    async def acquire(self) -> bool:
        """
        获取处理权限

        Returns:
            bool: 是否成功获取权限
        """
        try:
            await asyncio.wait_for(self.semaphore.acquire(), timeout=0.1)
            return True
        except asyncio.TimeoutError:
            return False

    def release(self):
        """释放处理权限"""
        self.semaphore.release()

    async def process_with_queue(
        self,
        func: Callable,
        *args,
        timeout: Optional[int] = None,
        **kwargs
    ) -> Any:
        """
        在队列中执行函数

        Args:
            func: 要执行的函数
            *args: 函数参数
            timeout: 超时时间 (秒)，None 使用配置值
            **kwargs: 关键字参数

        Returns:
            Any: 函数执行结果

        Raises:
            asyncio.QueueFull: 队列满时抛出
            asyncio.TimeoutError: 超时时抛出
        """
        timeout = timeout or self.config.timeout_seconds

        # 检查队列是否已满
        if self.queue.full():
            self.status.total_rejected += 1
            raise asyncio.QueueFull(f"队列已满 (max_size={self.config.max_size})")

        # 获取处理权限
        acquired = await self.acquire()
        if not acquired:
            # 排队等待
            self.status.pending += 1
            await self.semaphore.acquire()
            self.status.pending -= 1

        try:
            self.status.processing += 1

            # 执行函数
            if asyncio.iscoroutinefunction(func):
                result = await asyncio.wait_for(
                    func(*args, **kwargs),
                    timeout=timeout
                )
            else:
                result = await asyncio.get_event_loop().run_in_executor(
                    None, lambda: func(*args, **kwargs)
                )

            self.status.processing -= 1
            self.status.total_processed += 1
            self.status.last_processed_time = datetime.now()

            return result

        except asyncio.TimeoutError:
            logger.warning(f"请求超时 (timeout={timeout}s)")
            raise
        except Exception as e:
            logger.error(f"执行出错：{e}")
            raise
        finally:
            self.release()

    def get_status(self) -> Dict:
        """获取队列状态"""
        return {
            "pending": self.status.pending,
            "processing": self.status.processing,
            "max_concurrent": self.config.max_concurrent,
            "max_size": self.config.max_size,
            "total_processed": self.status.total_processed,
            "total_rejected": self.status.total_rejected,
        }


class QueueManager:
    """队列管理器 - 管理多个服务的队列"""

    def __init__(self):
        """初始化队列管理器"""
        self.queues: Dict[str, ServiceQueue] = {}
        self._lock = asyncio.Lock()

    def add_queue(self, service_name: str, config: ServiceQueueConfig):
        """
        添加服务队列

        Args:
            service_name: 服务名称
            config: 队列配置
        """
        self.queues[service_name] = ServiceQueue(config)
        logger.info(f"添加 {service_name} 队列：max_concurrent={config.max_concurrent}, "
                    f"max_size={config.max_size}, timeout={config.timeout_seconds}s")

    def get_queue(self, service_name: str) -> Optional[ServiceQueue]:
        """
        获取服务队列

        Args:
            service_name: 服务名称

        Returns:
            Optional[ServiceQueue]: 服务队列，不存在则返回 None
        """
        return self.queues.get(service_name)

    async def process_request(
        self,
        service_name: str,
        func: Callable,
        *args,
        timeout: Optional[int] = None,
        **kwargs
    ) -> Any:
        """
        在指定服务的队列中处理请求

        Args:
            service_name: 服务名称
            func: 要执行的函数
            *args: 函数参数
            timeout: 超时时间 (秒)
            **kwargs: 关键字参数

        Returns:
            Any: 函数执行结果

        Raises:
            ValueError: 服务队列不存在
            asyncio.QueueFull: 队列满
            asyncio.TimeoutError: 超时
        """
        queue = self.get_queue(service_name)
        if not queue:
            raise ValueError(f"服务队列不存在：{service_name}")

        return await queue.process_with_queue(func, *args, timeout=timeout, **kwargs)

    def get_status(self) -> Dict:
        """获取所有队列状态"""
        return {
            name: queue.get_status()
            for name, queue in self.queues.items()
        }

    def get_aggregated_status(self) -> Dict:
        """获取聚合状态"""
        total_pending = sum(q.status.pending for q in self.queues.values())
        total_processing = sum(q.status.processing for q in self.queues.values())
        return {
            "pending": total_pending,
            "processing": total_processing,
        }


# 全局队列管理器实例
_queue_manager: Optional[QueueManager] = None


def init_queue_manager(service_configs: Dict[str, ServiceQueueConfig]) -> QueueManager:
    """
    初始化全局队列管理器

    Args:
        service_configs: 服务队列配置字典

    Returns:
        QueueManager: 队列管理器实例
    """
    global _queue_manager
    _queue_manager = QueueManager()

    for service_name, config in service_configs.items():
        _queue_manager.add_queue(service_name, config)

    return _queue_manager


def get_queue_manager() -> Optional[QueueManager]:
    """获取全局队列管理器实例"""
    return _queue_manager



#!/usr/bin/env python3
"""
PI-LLM-Server 服务管理工具

支持启动、停止、重启服务，查看服务状态等功能。

使用方法:
    # 启动所有子服务
    python service_manager.py start --all

    # 启动单个服务
    python service_manager.py start embedding

    # 启动子服务 + 统一网关
    python service_manager.py start --with-gateway

    # 停止所有服务
    python service_manager.py stop --all

    # 查看服务状态
    python service_manager.py status

    # 重启服务
    python service_manager.py restart embedding
"""

import argparse
import os
import sys
import signal
import time
import subprocess
import yaml
import shutil
from pathlib import Path
from typing import Dict, Optional, Any

# 默认配置目录和文件
DEFAULT_CONFIG_DIR = Path.home() / ".config" / "pi-llm-server"
DEFAULT_CONFIG_FILE = DEFAULT_CONFIG_DIR / "config.yaml"

# 项目根目录（用于查找示例配置文件）
PROJECT_ROOT = Path(__file__).parent.parent.parent
EXAMPLE_CONFIG_FILE = PROJECT_ROOT / "examples" / "config.example.yaml"

# 服务配置（默认值，可从配置文件覆盖）
# 只包含后台子服务，不包含 gateway
SERVICE_CONFIG = {
    'embedding': {
        'script': 'embedding_server.py',
        'port': 8091,
        'name': 'Embedding Server',
    },
    'asr': {
        'script': 'asr_server.py',
        'port': 8092,
        'name': 'ASR Server',
    },
    'reranker': {
        'script': 'reranker_server.py',
        'port': 8093,
        'name': 'Reranker Server',
    },
    'mineru': {
        'script': 'mineru_server.sh',
        'port': 8094,
        'name': 'MinerU Server',
    },
}

# 用户目录
LOG_DIR = Path.home() / '.cache' / 'pi-llm-server' / 'logs'
PID_DIR = Path.home() / '.cache' / 'pi-llm-server' / 'pids'

# 确保目录存在
LOG_DIR.mkdir(parents=True, exist_ok=True)
PID_DIR.mkdir(parents=True, exist_ok=True)


def ensure_config_exists() -> Path:
    """确保配置文件存在，不存在则自动创建"""
    if not DEFAULT_CONFIG_DIR.exists():
        DEFAULT_CONFIG_DIR.mkdir(parents=True, exist_ok=True)
        print(f"创建配置目录：{DEFAULT_CONFIG_DIR}")

    if not DEFAULT_CONFIG_FILE.exists():
        # 从项目目录复制示例配置
        if EXAMPLE_CONFIG_FILE.exists():
            shutil.copy2(str(EXAMPLE_CONFIG_FILE), str(DEFAULT_CONFIG_FILE))
            print(f"创建默认配置文件：{DEFAULT_CONFIG_FILE}")
            print("请修改配置文件后重新启动")
            print(f"配置文件位置：{DEFAULT_CONFIG_FILE}")
        else:
            print(f"错误：找不到示例配置文件 {EXAMPLE_CONFIG_FILE}")
            print("请手动创建配置文件或检查项目结构")
            sys.exit(1)

    return DEFAULT_CONFIG_FILE


def load_config() -> Dict[str, Any]:
    """加载配置文件，如果不存在则自动创建"""
    ensure_config_exists()

    try:
        with open(DEFAULT_CONFIG_FILE, 'r', encoding='utf-8') as f:
            return yaml.safe_load(f) or {}
    except Exception as e:
        print(f"警告：配置文件加载失败：{e}")
        return {}


def get_service_config(service_name: str, config: Dict[str, Any]) -> Dict[str, Any]:
    """从配置文件获取服务配置，合并默认配置"""
    default = SERVICE_CONFIG.get(service_name, {})

    services_cfg = config.get('services', {})
    service_cfg = services_cfg.get(service_name, {})

    # 合并配置
    merged = {**default}

    if service_cfg.get('base_url'):
        # 从 base_url 提取端口
        import re
        match = re.search(r':(\d+)$', service_cfg['base_url'])
        if match:
            merged['port'] = int(match.group(1))

    # MinerU 特有配置
    if service_name == 'mineru':
        mineru_config = service_cfg.get('config', {})
        merged['vram'] = mineru_config.get('vram', '9000')
        merged['model_source'] = mineru_config.get('model_source', 'modelscope')
        merged['working_directory'] = service_cfg.get('working_directory', str(Path(__file__).parent))

    # 解析模型路径（展开 ~ 和环境变量）
    if 'models' in service_cfg:
        for model in service_cfg['models']:
            if 'path' in model:
                model['path'] = os.path.expanduser(os.path.expandvars(model['path']))

    return merged


def get_log_dir() -> Path:
    """获取日志目录"""
    return LOG_DIR


def get_pid_dir() -> Path:
    """获取 PID 目录"""
    return PID_DIR


def get_pid_file(service_name: str) -> Path:
    """获取服务的 PID 文件路径"""
    return get_pid_dir() / f"{service_name}.pid"


def get_log_file(service_name: str) -> Path:
    """获取服务的日志文件路径"""
    return get_log_dir() / f"{service_name}.log"


def is_service_running(service_name: str) -> bool:
    """检查服务是否正在运行"""
    config = SERVICE_CONFIG.get(service_name)
    if not config:
        return False

    port = config['port']

    # 方法 1: 检查 PID 文件
    pid_file = get_pid_file(service_name)
    if pid_file.exists():
        try:
            pid = int(pid_file.read_text().strip())
            os.kill(pid, 0)  # 检查进程是否存在
            return True
        except (ValueError, OSError):
            pid_file.unlink(missing_ok=True)

    # 方法 2: 检查端口
    import socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        result = sock.connect_ex(('127.0.0.1', port))
        sock.close()
        return result == 0
    except Exception:
        return False


def get_service_pid(service_name: str) -> Optional[int]:
    """获取服务的 PID"""
    pid_file = get_pid_file(service_name)
    if pid_file.exists():
        try:
            return int(pid_file.read_text().strip())
        except ValueError:
            return None
    return None


def start_service(service_name: str, background: bool = True, config: Dict[str, Any] = None) -> bool:
    """启动单个服务"""
    if config is None:
        config = load_config()

    service_config = get_service_config(service_name, config)

    if not service_config:
        print(f"错误：未知服务 '{service_name}'")
        return False

    if is_service_running(service_name):
        print(f"✓ {service_config['name']} 已在运行")
        return True

    script_dir = Path(__file__).parent
    log_file = get_log_file(service_name)
    pid_file = get_pid_file(service_name)

    print(f"启动 {service_config['name']}...", end=" ")

    try:
        if service_config['script'].endswith('.sh'):
            # Shell 脚本（MinerU）- 传递命令行参数
            working_dir = service_config.get('working_directory', str(script_dir))

            cmd = [
                'bash', script_dir / service_config['script'], 'start',
                '--host', '0.0.0.0',
                '--port', str(service_config['port']),
                '--vram', str(service_config.get('vram', '9000')),
                '--model-source', str(service_config.get('model_source', 'modelscope')),
                '--python-path', sys.executable,  # 使用当前环境的 Python
            ]
            # 切换到工作目录
            script_dir = Path(working_dir)

        else:
            # Python 脚本（Embedding/ASR/Reranker）- 使用当前环境的 Python
            cmd = [sys.executable, script_dir / service_config['script']]

            # 根据服务类型添加特定参数
            if service_name == 'embedding':
                # embedding 参数从配置读取
                models = config.get('services', {}).get('embedding', {}).get('models', [])
                if models:
                    model = models[0]
                    # 解析模型路径（展开 ~ 和环境变量）
                    model_path = os.path.expanduser(os.path.expandvars(model.get('path', '')))
                    cmd.extend(['--model-path', model_path])
                    cmd.extend(['--device', model.get('device', 'cpu')])

            elif service_name == 'asr':
                # ASR 参数从配置读取
                models = config.get('services', {}).get('asr', {}).get('models', [])
                if models:
                    model = models[0]
                    # 解析模型路径（展开 ~ 和环境变量）
                    model_path = os.path.expanduser(os.path.expandvars(model.get('path', '')))
                    cmd.extend(['--model-path', model_path])

            elif service_name == 'reranker':
                # Reranker 参数从配置读取
                models = config.get('services', {}).get('reranker', {}).get('models', [])
                if models:
                    model = models[0]
                    # 解析模型路径（展开 ~ 和环境变量）
                    model_path = os.path.expanduser(os.path.expandvars(model.get('path', '')))
                    cmd.extend(['--model-path', model_path])
                    cmd.extend(['--device', model.get('device', 'cpu')])

        if background:
            # 后台运行
            with open(log_file, 'a') as f:
                proc = subprocess.Popen(
                    cmd,
                    stdout=f,
                    stderr=f,
                    cwd=str(script_dir),
                    start_new_session=True,
                )
            # 写入 PID 文件（MinerU 除外，它有自己的 PID 管理）
            if service_name != 'mineru':
                pid_file.write_text(str(proc.pid))
            print(f"进程 ID: {proc.pid}")
        else:
            # 前台运行
            proc = subprocess.Popen(cmd, cwd=str(script_dir))
            pid_file.write_text(str(proc.pid))
            proc.wait()
            return True

        # 等待服务启动
        time.sleep(3)

        if is_service_running(service_name):
            print(f"✓ {service_config['name']} 启动成功")
            return True
        else:
            print(f"✗ {service_config['name']} 启动失败，请查看日志：{log_file}")
            return False

    except Exception as e:
        print(f"✗ {service_config['name']} 启动异常：{e}")
        return False


def stop_service(service_name: str) -> bool:
    """停止单个服务"""
    config = SERVICE_CONFIG.get(service_name)
    if not config:
        print(f"错误：未知服务 '{service_name}'")
        return False

    if not is_service_running(service_name):
        print(f"✓ {config['name']} 未运行")
        return True

    pid = get_service_pid(service_name)
    if pid:
        try:
            os.kill(pid, signal.SIGTERM)
            print(f"发送 SIGTERM 到 {config['name']} (PID: {pid})")

            # 等待进程终止
            for _ in range(10):
                time.sleep(0.5)
                try:
                    os.kill(pid, 0)
                except OSError:
                    break
            else:
                # 进程仍在运行，强制终止
                os.kill(pid, signal.SIGKILL)
                print(f"发送 SIGKILL 到 {config['name']}")

            # 删除 PID 文件
            pid_file = get_pid_file(service_name)
            pid_file.unlink(missing_ok=True)

            print(f"✓ {config['name']} 已停止")
            return True

        except ProcessLookupError:
            print(f"✓ {config['name']} 已停止")
            return True
        except Exception as e:
            print(f"✗ 停止 {config['name']} 失败：{e}")
            return False
    else:
        # 没有 PID 文件，尝试通过端口检查找到进程
        print(f"警告：未找到 {config['name']} 的 PID 文件")
        return False


def restart_service(service_name: str, config: Dict[str, Any] = None) -> bool:
    """重启服务"""
    if config is None:
        config = load_config()
    stop_service(service_name)
    time.sleep(1)
    return start_service(service_name, config=config)


def show_status():
    """显示所有服务状态"""
    print("=" * 60)
    print("PI-LLM-Server 后台服务状态")
    print("=" * 60)
    print()

    for name, cfg in SERVICE_CONFIG.items():
        status = "运行中" if is_service_running(name) else "已停止"
        pid = get_service_pid(name)
        pid_str = f"(PID: {pid})" if pid else ""
        symbol = "✓" if is_service_running(name) else "✗"
        print(f"  {symbol} {cfg['name']:20s} {status:10s} 端口：{cfg['port']:5d} {pid_str}")

    print()
    print("=" * 60)

    # 显示目录信息
    print(f"日志目录：{get_log_dir()}")
    print(f"PID 目录：{get_pid_dir()}")
    config_path = ensure_config_exists()
    print(f"配置文件：{config_path}")


def start_all():
    """启动所有后台服务"""
    config = load_config()

    print("正在启动所有后台服务...")
    print()

    # 启动子服务
    for name in ['embedding', 'asr', 'reranker', 'mineru']:
        start_service(name, config=config)


def stop_all():
    """停止所有后台服务"""
    print("正在停止所有后台服务...")
    print()

    # 停止子服务（按依赖顺序：先启动的后停止）
    for name in ['mineru', 'reranker', 'asr', 'embedding']:
        stop_service(name)


def main():
    parser = argparse.ArgumentParser(
        description="PI-LLM-Server 后台服务管理工具",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
示例:
  %(prog)s start --all           # 启动所有后台服务
  %(prog)s start embedding       # 启动单个服务
  %(prog)s stop --all            # 停止所有服务
  %(prog)s stop embedding        # 停止单个服务
  %(prog)s restart embedding     # 重启服务
  %(prog)s status                # 查看服务状态

注意：本工具只管理后台子服务 (embedding/asr/reranker/mineru)，
      网关服务请使用 python -m pi_llm_server 启动
        """
    )

    subparsers = parser.add_subparsers(dest='command', help='命令')

    # start 命令
    start_parser = subparsers.add_parser('start', help='启动服务')
    start_parser.add_argument('service', nargs='?', help='服务名称 (embedding/asr/reranker/mineru)')
    start_parser.add_argument('--all', action='store_true', help='启动所有后台服务')

    # stop 命令
    stop_parser = subparsers.add_parser('stop', help='停止服务')
    stop_parser.add_argument('service', nargs='?', help='服务名称')
    stop_parser.add_argument('--all', action='store_true', help='停止所有服务')

    # restart 命令
    restart_parser = subparsers.add_parser('restart', help='重启服务')
    restart_parser.add_argument('service', help='服务名称')

    # status 命令
    status_parser = subparsers.add_parser('status', help='查看服务状态')

    args = parser.parse_args()

    if args.command == 'start':
        if args.all:
            start_all()
        elif args.service:
            start_service(args.service)
        else:
            start_all()

    elif args.command == 'stop':
        if args.all:
            stop_all()
        elif args.service:
            stop_service(args.service)
        else:
            stop_all()

    elif args.command == 'restart':
        if args.service:
            restart_service(args.service)
        else:
            print("错误：restart 命令需要指定服务名称")
            sys.exit(1)

    elif args.command == 'status':
        show_status()

    else:
        parser.print_help()
        sys.exit(1)


if __name__ == '__main__':
    main()



"""
PI-LLM-Server - 统一 LLM 服务网关

集成 Embedding、ASR、Reranker、MinerU 四个子服务，
提供统一的 API 入口、请求队列、认证管理、健康监控等功能。
"""

from importlib.metadata import version

__version__ = version("pi-llm-server")
__author__ = "PI-Lab Team"

from .config import ConfigManager, Config, ServiceConfig
from .auth import AuthManager
from .queue_manager import QueueManager, ServiceQueueConfig

__all__ = [
    "ConfigManager",
    "Config",
    "ServiceConfig",
    "AuthManager",
    "QueueManager",
    "ServiceQueueConfig",
]



#!/usr/bin/env python3
"""
PI-LLM-Server 完整测试脚本

测试流程:
1. 启动所有子服务 (embedding, asr, reranker, mineru)
2. 启动 pi-llm-server 网关
3. 等待所有服务启动完成
4. 测试所有 API 端点
5. 输出测试结果
6. 停止所有服务
"""

import subprocess
import time
import sys
import os
import json
import httpx
from pathlib import Path

# 配置
GATEWAY_URL = "http://127.0.0.1:8090"
SERVICE_PORTS = {
    'embedding': 8091,
    'asr': 8092,
    'reranker': 8093,
    'mineru': 8094,
}
HEALTH_CHECK_TIMEOUT = 180  # 等待服务启动的最大时间 (秒)
API_CHECK_TIMEOUT = 30  # 单个 API 调用超时 (秒)
TEST_TOKEN = "sk-5f8b839908d14561590b70227c72ca86"  # 测试 Token

# 测试数据目录
DATA_DIR = Path(__file__).parent.parent / "data"

# 项目根目录
PROJECT_ROOT = Path(__file__).parent.parent

#  ANSI 颜色代码
COLORS = {
    "green": "\033[92m",
    "red": "\033[91m",
    "yellow": "\033[93m",
    "blue": "\033[94m",
    "reset": "\033[0m",
    "bold": "\033[1m",
}


def colored(text, color):
    """返回彩色文本"""
    return f"{COLORS.get(color, '')}{text}{COLORS['reset']}"


def print_header(text):
    """打印标题"""
    print(f"\n{COLORS['bold']}{'=' * 60}{COLORS['reset']}")
    print(f"{COLORS['bold']}{text}{COLORS['reset']}")
    print(f"{COLORS['bold']}{'=' * 60}{COLORS['reset']}")


def print_status(name, status, detail=""):
    """打印测试状态"""
    color = "green" if status == "PASS" else ("red" if status == "FAIL" else "yellow")
    status_str = f"[{status}]"
    print(f"  {status_str:8} {name}", end="")
    if detail:
        print(f" - {detail}")
    else:
        print()


def check_port_open(port, timeout=5):
    """检查端口是否开放"""
    import socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(timeout)
    try:
        result = sock.connect_ex(('127.0.0.1', port))
        sock.close()
        return result == 0
    except Exception:
        return False


def wait_for_port(port, max_wait=60, service_name=""):
    """等待端口开放"""
    start_time = time.time()
    while time.time() - start_time < max_wait:
        if check_port_open(port):
            print(f"  {colored('✓', 'green')} {service_name} 已启动 (端口 {port})")
            return True
        elapsed = int(time.time() - start_time)
        print(f"  等待 {service_name}... ({elapsed}s/{max_wait}s)", end="\r")
        time.sleep(2)
    print(f"\n  {colored(f'✗ {service_name} 启动超时!', 'red')}")
    return False


class ServiceTest:
    """服务测试类"""

    def __init__(self, base_url, token):
        self.base_url = base_url
        self.token = token
        self.client = httpx.Client(
            base_url=base_url,
            timeout=httpx.Timeout(API_CHECK_TIMEOUT),
            headers={"Authorization": f"Bearer {token}"}
        )
        self.results = []

    def wait_for_gateway(self, max_wait=HEALTH_CHECK_TIMEOUT):
        """等待网关启动"""
        print_header("等待网关服务启动")
        start_time = time.time()

        while time.time() - start_time < max_wait:
            try:
                response = self.client.get("/health", timeout=5.0)
                if response.status_code == 200:
                    data = response.json()
                    print(f"  {colored('✓', 'green')} 网关已启动！")
                    print(f"  健康状态：{json.dumps(data, indent=2, ensure_ascii=False)}")
                    return True
            except httpx.ConnectError:
                pass
            except Exception as e:
                print(f"  检查健康状态时出错：{e}")

            elapsed = int(time.time() - start_time)
            print(f"  等待网关启动... ({elapsed}s/{max_wait}s)", end="\r")
            time.sleep(2)

        print(f"\n  {colored('网关启动超时!', 'red')}")
        return False

    def test_endpoint(self, method, path, name, **kwargs):
        """测试单个 API 端点"""
        try:
            func = getattr(self.client, method.lower())
            response = func(path, **kwargs)

            if response.status_code < 300:
                print_status(name, "PASS", f"status={response.status_code}")
                self.results.append((name, "PASS", response.status_code))
                return response
            else:
                print_status(name, "FAIL", f"status={response.status_code}, body={response.text[:200]}")
                self.results.append((name, "FAIL", response.status_code))
                return None

        except httpx.TimeoutException:
            print_status(name, "FAIL", "timeout")
            self.results.append((name, "FAIL", "timeout"))
            return None
        except Exception as e:
            print_status(name, "FAIL", str(e))
            self.results.append((name, "FAIL", str(e)))
            return None

    def run_all_tests(self):
        """运行所有测试"""
        print_header("开始 API 测试")

        # 1. 根路径
        print("\n[1] 基础端点测试")
        self.test_endpoint("get", "/", "根路径 /")

        # 2. 健康检查
        print("\n[2] 健康检查端点")
        health_resp = self.test_endpoint("get", "/health", "健康检查 /health")
        if health_resp:
            print(f"      响应：{json.dumps(health_resp.json(), indent=2, ensure_ascii=False)}")

        # 3. 状态详情
        print("\n[3] 状态详情端点")
        status_resp = self.test_endpoint("get", "/status", "状态详情 /status")
        if status_resp:
            status_data = status_resp.json()
            print(f"      响应预览：{json.dumps(status_data, indent=2, ensure_ascii=False)[:500]}...")

        # 4. 模型列表
        print("\n[4] 模型列表端点")
        models_resp = self.test_endpoint("get", "/v1/models", "模型列表 /v1/models")
        if models_resp:
            models_data = models_resp.json()
            models = models_data.get("data", [])
            print(f"      可用模型数量：{len(models)}")
            for m in models[:5]:
                print(f"        - {m.get('id', 'unknown')} ({m.get('service', 'unknown')})")

        # 5. Embedding 服务测试
        print("\n[5] Embedding 服务测试")

        # 5.1 测试 embedding 向量生成
        embedding_request = {
            "input": "人工智能是计算机科学的一个分支",
            "model": "unsloth/Qwen3-Embedding-0.6B"
        }
        emb_resp = self.test_endpoint(
            "post", "/v1/embeddings",
            "Embedding 向量生成",
            json=embedding_request
        )
        if emb_resp:
            data = emb_resp.json()
            if "data" in data and len(data["data"]) > 0:
                vec = data["data"][0].get("embedding", [])
                print(f"      向量维度：{len(vec)}")

        # 5.2 测试相似度计算
        similarity_request = {
            "text1": "人工智能是计算机科学的一个分支",
            "text2": "AI 技术正在改变各行各业"
        }
        sim_resp = self.test_endpoint(
            "post", "/v1/similarity",
            "相似度计算",
            json=similarity_request
        )
        if sim_resp:
            print(f"      相似度：{sim_resp.json().get('similarity', 'N/A')}")

        # 5.3 使用测试数据进行批量测试
        print("\n[6] Embedding 批量测试 (使用 data/ 测试数据)")
        test_data_file = DATA_DIR / "test_data_similarity.json"
        if test_data_file.exists():
            with open(test_data_file) as f:
                test_data = json.load(f)

            documents = test_data.get("documents", [])[:3]  # 取前 3 个
            queries = test_data.get("queries", [])[:2]  # 取前 2 个

            print(f"      测试文档数：{len(documents)}")
            print(f"      测试查询数：{len(queries)}")

            # 批量生成 embedding
            if documents:
                batch_request = {"input": documents}
                batch_resp = self.test_endpoint(
                    "post", "/v1/embeddings",
                    "批量 Embedding 生成",
                    json=batch_request
                )
                if batch_resp:
                    data = batch_resp.json()
                    vecs = data.get("data", [])
                    print(f"      生成向量数：{len(vecs)}")

            # 搜索测试
            for query in queries:
                search_request = {"input": query}
                search_resp = self.test_endpoint(
                    "post", "/v1/embeddings",
                    f"搜索查询：{query[:20]}...",
                    json=search_request
                )
        else:
            print_status("测试数据文件不存在", "SKIP")

        # 7. ASR 服务测试
        print("\n[7] ASR 服务测试")
        audio_file = DATA_DIR / "audio_s.mp3"
        if audio_file.exists():
            print(f"      测试音频文件：{audio_file}")
            audio_size = audio_file.stat().st_size / (1024 * 1024)
            print(f"      文件大小：{audio_size:.2f} MB")

            with open(audio_file, "rb") as f:
                files = {"file": ("audio_s.mp3", f, "audio/mp3")}
                data = {"model": "Qwen/Qwen3-ASR-1.7B"}

                asr_start = time.time()
                asr_resp = self.test_endpoint(
                    "post", "/v1/audio/transcriptions",
                    "语音识别 (短音频)",
                    files=files,
                    data=data,
                    timeout=120.0  # ASR 可能耗时较长，增加超时
                )
                asr_elapsed = time.time() - asr_start

                if asr_resp:
                    result = asr_resp.json()
                    print(f"      识别结果：{result.get('text', 'N/A')[:100]}...")
                    print(f"      耗时：{asr_elapsed:.2f}s")
        else:
            print_status("音频文件不存在", "SKIP")

        # 8. Reranker 服务测试
        print("\n[8] Reranker 服务测试")
        rerank_request = {
            "query": "什么是人工智能",
            "documents": [
                "人工智能是计算机科学的一个分支，致力于创建智能系统",
                "机器学习使用算法让计算机从数据中学习",
                "深度学习是机器学习的子集，使用神经网络"
            ]
        }
        rerank_resp = self.test_endpoint(
            "post", "/v1/rerank",
            "Reranker 重排序",
            json=rerank_request
        )
        if rerank_resp:
            result = rerank_resp.json()
            results = result.get("results", [])
            print(f"      返回结果数：{len(results)}")
            if results:
                for i, r in enumerate(results[:2]):
                    print(f"        [{i}] score={r.get('relevance_score', r.get('score', 'N/A')):.4f}")

        # 9. MinerU 服务测试
        print("\n[9] MinerU 服务测试")
        pdf_file = DATA_DIR / "InfoLOD.pdf"
        if pdf_file.exists():
            print(f"      测试 PDF 文件：{pdf_file}")

            # 获取文件大小
            file_size_mb = pdf_file.stat().st_size / (1024 * 1024)
            print(f"      文件大小：{file_size_mb:.2f} MB")

            # MinerU 使用 /file_parse 端点
            with open(pdf_file, "rb") as f:
                files = {"file": ("InfoLOD.pdf", f, "application/pdf")}
                data = {
                    "backend": "pipeline",
                    "parse_method": "auto",
                    "lang_list": "ch",
                    "formula_enable": "true",
                    "table_enable": "true",
                    "return_md": "true",
                    "return_images": "false",  # 不返回图片以加快测试
                }

                mineru_start = time.time()
                mineru_resp = self.test_endpoint(
                    "post", "/file_parse",
                    "MinerU PDF 解析",
                    files=files,
                    data=data,
                    timeout=60.0  # PDF 解析耗时，增加超时
                )
                mineru_elapsed = time.time() - mineru_start

                if mineru_resp:
                    try:
                        result = mineru_resp.json()
                        status = result.get("status", "N/A")
                        print(f"      解析状态：{status}")
                        if "data" in result and result["data"]:
                            text = result["data"].get("markdown", "")
                            print(f"      文本长度：{len(text)}")
                            print(f"      文本预览：{text[:200]}...")
                        elif "message" in result:
                            print(f"      消息：{result.get('message', '')[:200]}")
                    except:
                        print(f"      响应内容：{mineru_resp.text[:200]}...")
                    print(f"      耗时：{mineru_elapsed:.2f}s")
        else:
            print_status("PDF 文件不存在", "SKIP")

        # 打印测试总结
        self.print_summary()

    def print_summary(self):
        """打印测试总结"""
        print_header("测试总结")

        passed = sum(1 for _, status, _ in self.results if status == "PASS")
        failed = sum(1 for _, status, _ in self.results if status in ("FAIL", "timeout"))
        skipped = sum(1 for _, status, _ in self.results if status == "SKIP")

        print(f"  总计：{len(self.results)} 项")
        print(f"  {colored('通过：' + str(passed), 'green')}")
        print(f"  {colored('失败：' + str(failed), 'red')}")
        if skipped > 0:
            print(f"  {colored('跳过：' + str(skipped), 'yellow')}")

        if failed > 0:
            print(f"\n  {colored('失败的测试:', 'red')}")
            for name, status, detail in self.results:
                if status in ("FAIL", "timeout"):
                    print(f"    - {name}: {detail}")

        print()


def start_sub_services():
    """启动所有子服务"""
    print_header("启动子服务")

    service_manager = PROJECT_ROOT / "pi_llm_server" / "launcher" / "service_manager.py"

    print(f"  使用 service_manager.py 启动子服务...")
    print(f"  服务列表：{list(SERVICE_PORTS.keys())}")

    # 检查子服务是否已经在运行
    running_services = []
    for name, port in SERVICE_PORTS.items():
        if check_port_open(port):
            running_services.append(name)
            print(f"  {colored('✓', 'yellow')} {name} 已在运行 (端口 {port})")

    if running_services:
        print(f"  已有服务：{running_services}")
        use_existing = input("  是否使用现有服务进行测试？(y/n): ").strip().lower()
        if use_existing == 'y':
            return True
        # 否则停止现有服务
        print("  正在停止现有服务...")
        subprocess.run(
            [sys.executable, str(service_manager), "stop", "--all"],
            cwd=PROJECT_ROOT
        )
        time.sleep(3)

    # 启动所有子服务
    print("\n  启动子服务...")
    result = subprocess.run(
        [sys.executable, str(service_manager), "start", "--all"],
        cwd=PROJECT_ROOT,
        capture_output=False
    )

    if result.returncode != 0:
        print(f"  {colored('✗ 子服务启动失败!', 'red')}")
        return False

    # 等待各服务端口就绪
    print("\n  等待服务端口就绪...")
    all_ready = True
    for name, port in SERVICE_PORTS.items():
        if not wait_for_port(port, max_wait=60, service_name=name):
            all_ready = False

    return all_ready


def start_gateway(config_path):
    """启动网关服务"""
    print_header("启动网关服务")

    cmd = [
        sys.executable, "-m", "pi_llm_server",
        "--config", str(config_path),
        "--log-level", "info"
    ]

    print(f"  启动命令：{' '.join(cmd)}")

    process = subprocess.Popen(
        cmd,
        cwd=str(PROJECT_ROOT),
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=True,
        bufsize=1
    )

    return process


def main():
    """主函数"""
    print_header("PI-LLM-Server 完整测试")
    print(f"  网关地址：{GATEWAY_URL}")
    print(f"  测试 Token: {TEST_TOKEN[:10]}...")
    print(f"  数据目录：{DATA_DIR}")
    print(f"  服务端口：{SERVICE_PORTS}")

    # 查找配置文件
    config_path = Path.home() / ".config" / "pi-llm-server" / "config.yaml"
    if not config_path.exists():
        print(f"  {colored(f'配置文件不存在：{config_path}', 'red')}")
        sys.exit(1)

    print(f"  配置文件：{config_path}")

    # 步骤 1: 启动子服务
    if not start_sub_services():
        print("\n  子服务启动失败，终止测试")
        sys.exit(1)

    print("\n  子服务已全部启动!")

    # 步骤 2: 启动网关
    print("\n[步骤 2] 启动网关服务...")
    gateway_process = start_gateway(config_path)

    # 步骤 3: 等待网关启动
    test = ServiceTest(GATEWAY_URL, TEST_TOKEN)
    if not test.wait_for_gateway():
        print("\n  网关启动失败，终止测试")
        gateway_process.terminate()
        stop_sub_services()
        sys.exit(1)

    # 步骤 4: 运行 API 测试
    test.run_all_tests()

    # 步骤 5: 停止服务
    print("\n[步骤 5] 停止服务...")
    print("  停止网关服务...")
    gateway_process.terminate()
    try:
        gateway_process.wait(timeout=10)
        print("  网关已停止")
    except subprocess.TimeoutExpired:
        print("  网关停止超时，强制终止")
        gateway_process.kill()

    stop_sub_services()

    print("\n" + "=" * 60)
    print("测试完成!")
    print("=" * 60)


def stop_sub_services():
    """停止所有子服务"""
    service_manager = PROJECT_ROOT / "pi_llm_server" / "launcher" / "service_manager.py"
    print("  正在停止子服务...")
    subprocess.run(
        [sys.executable, str(service_manager), "stop", "--all"],
        cwd=PROJECT_ROOT
    )
    print("  子服务已停止")


if __name__ == "__main__":
    main()



"""
认证模块测试
"""
import pytest
from pathlib import Path
from pi_llm_server.config import ConfigManager
from pi_llm_server.auth import AuthManager


EXAMPLE_CONFIG = Path(__file__).parent.parent / "examples" / "config.example.yaml"


class TestAuthManager:
    """认证管理器测试"""

    def test_auth_manager_creation(self):
        """测试认证管理器创建"""
        auth = AuthManager(tokens=[])
        assert auth.tokens == set()

    def test_auth_manager_with_tokens(self):
        """测试带 token 的认证管理器"""
        tokens = ["token1", "token2", "token3"]
        auth = AuthManager(tokens=tokens)
        assert len(auth.tokens) == 3
        assert "token1" in auth.tokens

    def test_validate_token(self):
        """测试 Token 验证"""
        tokens = ["valid-token-1", "valid-token-2"]
        auth = AuthManager(tokens=tokens)

        # 有效 token
        assert auth.validate_token("valid-token-1") is True
        assert auth.validate_token("valid-token-2") is True

        # 无效 token
        assert auth.validate_token("invalid-token") is False

    def test_load_from_config(self):
        """测试从配置文件加载"""
        config = ConfigManager(str(EXAMPLE_CONFIG))
        auth = AuthManager(tokens=config.get_auth_tokens())

        assert "sk-admin-token-001" in auth.tokens
        assert "sk-embedding-token-001" in auth.tokens



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
日志工具模块

提供结构化日志配置和日志记录器。
"""

import os
import sys
import logging
from logging.handlers import RotatingFileHandler
from datetime import datetime
from pathlib import Path
from typing import Optional


class JsonFormatter(logging.Formatter):
    """JSON 格式化器 - 可选使用"""

    def format(self, record: logging.LogRecord) -> str:
        """格式化为 JSON 格式"""
        import json

        log_data = {
            "timestamp": datetime.utcnow().isoformat() + "Z",
            "level": record.levelname,
            "service": getattr(record, "service", "pi-llm-server"),
            "message": record.getMessage(),
        }

        # 添加可选字段
        if hasattr(record, "endpoint"):
            log_data["endpoint"] = record.endpoint
        if hasattr(record, "method"):
            log_data["method"] = record.method
        if hasattr(record, "status"):
            log_data["status"] = record.status
        if hasattr(record, "latency_ms"):
            log_data["latency_ms"] = record.latency_ms
        if hasattr(record, "client_ip"):
            log_data["client_ip"] = record.client_ip
        if hasattr(record, "model"):
            log_data["model"] = record.model
        if hasattr(record, "queue_wait_ms"):
            log_data["queue_wait_ms"] = record.queue_wait_ms

        # 添加错误信息
        if record.exc_info:
            log_data["exception"] = self.formatException(record.exc_info)

        return json.dumps(log_data, ensure_ascii=False)


def setup_logging(
    service_name: str = "pi-llm-server",
    log_level: str = "info",
    log_dir: Optional[str] = None,
    use_json: bool = False,
) -> logging.Logger:
    """
    配置日志系统

    Args:
        service_name: 服务名称
        log_level: 日志级别 (debug, info, warning, error)
        log_dir: 日志目录 (None 则不创建文件 handler)
        use_json: 是否使用 JSON 格式

    Returns:
        logging.Logger: 配置好的 logger
    """
    # 创建 logger
    logger = logging.getLogger(service_name)
    logger.setLevel(getattr(logging, log_level.upper()))

    # 清除已有的 handlers
    logger.handlers = []

    # 创建 formatter
    if use_json:
        formatter = JsonFormatter()
    else:
        formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S"
        )

    # 控制台 handler
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setLevel(logging.INFO)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)

    # 文件 handler (可选)
    if log_dir:
        os.makedirs(log_dir, exist_ok=True)
        log_file = os.path.join(log_dir, f"{service_name}.log")

        # 旋转文件 handler (最大 10MB，保留 5 个文件)
        file_handler = RotatingFileHandler(
            log_file,
            maxBytes=10 * 1024 * 1024,  # 10MB
            backupCount=5,
            encoding="utf-8"
        )
        file_handler.setLevel(getattr(logging, log_level.upper()))
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)

    return logger


def get_logger(name: Optional[str] = None) -> logging.Logger:
    """
    获取 logger 实例

    Args:
        name: logger 名称 (默认使用模块名)

    Returns:
        logging.Logger: logger 实例
    """
    if name:
        return logging.getLogger(name)
    return logging.getLogger()


# 默认 logger 实例
default_logger: Optional[logging.Logger] = None


def init_default_logging(
    service_name: str = "pi-llm-server",
    log_level: str = "info",
    log_dir: Optional[str] = None,
) -> logging.Logger:
    """
    初始化默认日志

    Args:
        service_name: 服务名称
        log_level: 日志级别
        log_dir: 日志目录

    Returns:
        logging.Logger: 默认 logger
    """
    global default_logger
    default_logger = setup_logging(service_name, log_level, log_dir)
    return default_logger


def get_default_logger() -> logging.Logger:
    """获取默认 logger"""
    global default_logger
    if not default_logger:
        default_logger = setup_logging()
    return default_logger



"""
服务模块测试
"""
import pytest
from pi_llm_server.config import ServiceConfig
from pi_llm_server.services import (
    EmbeddingService,
    ASRService,
    RerankerService,
    MinerUService,
)


class TestEmbeddingService:
    """Embedding 服务测试"""

    def test_service_creation(self):
        """测试服务创建"""
        config = ServiceConfig(
            base_url="http://localhost:8091",
            timeout_seconds=60,
            max_retries=3,
        )
        service = EmbeddingService(config=config)
        assert service.base_url == "http://localhost:8091"
        assert service.timeout == 60


class TestASRService:
    """ASR 服务测试"""

    def test_service_creation(self):
        """测试服务创建"""
        config = ServiceConfig(
            base_url="http://localhost:8092",
            timeout_seconds=300,
            max_retries=2,
        )
        service = ASRService(config=config)
        assert service.base_url == "http://localhost:8092"
        assert service.timeout == 300


class TestRerankerService:
    """Reranker 服务测试"""

    def test_service_creation(self):
        """测试服务创建"""
        config = ServiceConfig(
            base_url="http://localhost:8093",
            timeout_seconds=120,
            max_retries=3,
        )
        service = RerankerService(config=config)
        assert service.base_url == "http://localhost:8093"
        assert service.timeout == 120


class TestMinerUService:
    """MinerU 服务测试"""

    def test_service_creation(self):
        """测试服务创建"""
        config = ServiceConfig(
            base_url="http://localhost:8094",
            timeout_seconds=600,
            max_retries=1,
        )
        service = MinerUService(config=config)
        assert service.base_url == "http://localhost:8094"
        assert service.timeout == 600



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Reranker 服务模块

提供文档相关性排序服务，包括:
- /v1/rerank: 文档重排序
"""

import httpx
import logging
from typing import List, Optional, Any
from fastapi import APIRouter, HTTPException, Body
from pydantic import BaseModel, Field

from ..config import ServiceConfig

logger = logging.getLogger(__name__)


# ============ 请求/响应模型 ============

class RerankRequest(BaseModel):
    """Rerank 请求"""
    query: str = Field(..., description="查询文本")
    documents: List[str] = Field(..., description="文档列表")
    top_n: Optional[int] = Field(default=None, ge=1, le=100, description="返回前 N 个结果")
    model: Optional[str] = Field(default=None, description="模型 ID")
    encoding_format: Optional[str] = Field(default=None, description="编码格式：float | base64")


class RerankResult(BaseModel):
    """Rerank 结果"""
    index: int
    document: str
    relevance_score: float


class RerankResponse(BaseModel):
    """Rerank 响应"""
    model: str
    results: List[RerankResult]
    usage: Optional[dict] = None


# ============ 服务类 ============

class RerankerService:
    """Reranker 服务 (Client + Router)"""

    def __init__(self, config: ServiceConfig):
        """
        初始化 Reranker 服务

        Args:
            config: 服务配置
        """
        self.config = config
        self.base_url = config.base_url
        self.timeout = config.timeout_seconds
        self.max_retries = config.max_retries

        # HTTP 客户端
        self.client = httpx.AsyncClient(
            base_url=self.base_url,
            timeout=httpx.Timeout(self.timeout),
            limits=httpx.Limits(max_keepalive_connections=10, max_connections=20)
        )

        # Router
        self.router = APIRouter(tags=["reranker"])
        self._register_routes()

        # 服务信息
        self.models = config.models if hasattr(config, 'models') else []

    async def rerank(
        self,
        query: str,
        documents: List[str],
        top_n: Optional[int] = None,
        model: Optional[str] = None,
        encoding_format: Optional[str] = None,
    ) -> dict:
        """
        调用 reranker 服务对文档进行相关性排序

        Args:
            query: 查询文本
            documents: 文档列表
            top_n: 返回前 N 个结果 (可选)
            model: 模型 ID (可选)
            encoding_format: 编码格式：float | base64 (可选)

        Returns:
            dict: rerank 响应

        Raises:
            HTTPException: 请求失败时抛出
        """
        request_data = {
            "query": query,
            "documents": documents,
        }
        if top_n:
            request_data["top_n"] = top_n
        if model:
            request_data["model"] = model
        if encoding_format:
            request_data["encoding_format"] = encoding_format

        for attempt in range(self.max_retries):
            try:
                response = await self.client.post(
                    "/v1/rerank",
                    json=request_data,
                )
                response.raise_for_status()
                return response.json()
            except httpx.TimeoutException as e:
                logger.warning(f"Rerank 请求超时 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=504, detail="Reranker 服务响应超时")
            except httpx.HTTPError as e:
                logger.error(f"Rerank 请求失败 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=502, detail=f"Reranker 服务错误：{str(e)}")

        return None

    async def health_check(self) -> dict:
        """
        健康检查

        Returns:
            dict: 健康状态
        """
        try:
            response = await self.client.get("/health", timeout=5.0)
            if response.status_code == 200:
                return {"status": "healthy", "latency_ms": response.elapsed.total_seconds() * 1000}
            else:
                return {"status": "unhealthy", "status_code": response.status_code}
        except Exception as e:
            return {"status": "unhealthy", "error": str(e)}

    async def get_models(self) -> List[dict]:
        """
        获取可用模型列表

        Returns:
            List[dict]: 模型列表
        """
        try:
            response = await self.client.get("/v1/models", timeout=5.0)
            if response.status_code == 200:
                data = response.json()
                # 添加服务信息
                models = data.get("data", [])
                for model in models:
                    model["service"] = "reranker"
                return models
            return []
        except Exception as e:
            logger.warning(f"获取模型列表失败：{e}")
            return []

    def _register_routes(self):
        """注册 FastAPI 路由"""

        @self.router.post("/v1/rerank", tags=["reranker"])
        async def rerank_endpoint(request: RerankRequest):
            """对文档进行相关性排序"""
            return await self.rerank(
                query=request.query,
                documents=request.documents,
                top_n=request.top_n,
                model=request.model,
            )

    async def close(self):
        """关闭 HTTP 客户端"""
        await self.client.aclose()


# ============ 全局实例 ============

reranker_service: Optional[RerankerService] = None


def init_reranker_service(config: ServiceConfig) -> RerankerService:
    """
    初始化 Reranker 服务

    Args:
        config: 服务配置

    Returns:
        RerankerService: 服务实例
    """
    global reranker_service
    reranker_service = RerankerService(config)
    logger.info(f"Reranker 服务初始化完成：{config.base_url}")
    return reranker_service


def get_reranker_service() -> RerankerService:
    """获取 Reranker 服务实例"""
    if not reranker_service:
        raise RuntimeError("Reranker 服务未初始化")
    return reranker_service



"""
工具模块
"""

from .logging import setup_logging, get_logger
from .exceptions import (
    PIException,
    AuthenticationError,
    ServiceUnavailableError,
    QueueFullError,
    TimeoutError,
)

__all__ = [
    "setup_logging",
    "get_logger",
    "PIException",
    "AuthenticationError",
    "ServiceUnavailableError",
    "QueueFullError",
    "TimeoutError",
]



#!/usr/bin/env python3
"""
PI-LLM-Server 命令行入口 - 统一网关 + 后台服务一站式启动
"""
import argparse
import sys
import os
import time
import subprocess
from pathlib import Path

# 导入 FastAPI 应用和配置
from pi_llm_server.server import app, initialize_services
from pi_llm_server.config import init_config, ConfigManager
from pi_llm_server.utils.logging import init_default_logging
import uvicorn

# 默认配置目录
DEFAULT_CONFIG_DIR = Path.home() / ".config" / "pi-llm-server"
DEFAULT_CONFIG_FILE = DEFAULT_CONFIG_DIR / "config.yaml"


def ensure_config_exists() -> Path:
    """确保配置文件存在，不存在则自动创建"""
    if not DEFAULT_CONFIG_DIR.exists():
        DEFAULT_CONFIG_DIR.mkdir(parents=True, exist_ok=True)
        print(f"创建配置目录：{DEFAULT_CONFIG_DIR}")

    if not DEFAULT_CONFIG_FILE.exists():
        # 从包内复制示例配置
        script_dir = Path(__file__).parent
        example_config = script_dir / "examples" / "config.example.yaml"
        if example_config.exists():
            import shutil
            shutil.copy2(example_config, DEFAULT_CONFIG_FILE)
            print(f"创建默认配置文件：{DEFAULT_CONFIG_FILE}")
            print("请修改配置文件后重新启动")
            print(f"配置文件位置：{DEFAULT_CONFIG_FILE}")
            sys.exit(1)
        else:
            print("错误：找不到示例配置文件 examples/config.example.yaml")
            sys.exit(1)

    return DEFAULT_CONFIG_FILE


def start_gateway_background(config_file: Path = None):
    """后台启动网关服务"""
    config_file = config_file or ensure_config_exists()

    log_dir = Path.home() / ".cache" / "pi-llm-server" / "logs"
    log_dir.mkdir(parents=True, exist_ok=True)
    log_file = log_dir / "gateway.log"

    pid_dir = Path.home() / ".cache" / "pi-llm-server" / "pids"
    pid_dir.mkdir(parents=True, exist_ok=True)
    pid_file = pid_dir / "gateway.pid"

    with open(log_file, 'a') as f:
        proc = subprocess.Popen(
            [sys.executable, '-m', 'pi_llm_server', '--config', str(config_file)],
            stdout=f,
            stderr=f,
            start_new_session=True,
        )
    pid_file.write_text(str(proc.pid))
    print(f"✓ 网关已启动 (PID: {proc.pid})")
    time.sleep(2)
    return True


def stop_gateway() -> bool:
    """停止网关服务"""
    import signal
    import socket

    pid_dir = Path.home() / ".cache" / "pi-llm-server" / "pids"
    pid_file = pid_dir / "gateway.pid"

    if pid_file.exists():
        try:
            pid = int(pid_file.read_text().strip())
            os.kill(pid, signal.SIGTERM)
            print(f"发送 SIGTERM 到网关 (PID: {pid})")

            # 等待进程终止
            for _ in range(10):
                time.sleep(0.5)
                try:
                    os.kill(pid, 0)
                except OSError:
                    break
            else:
                os.kill(pid, signal.SIGKILL)
                print(f"发送 SIGKILL 到网关")

            pid_file.unlink()
            print("✓ 网关已停止")
            return True

        except ProcessLookupError:
            pid_file.unlink(missing_ok=True)
            print("✓ 网关已停止")
            return True
        except Exception as e:
            print(f"✗ 停止网关失败：{e}")
            return False

    # 尝试通过端口检查
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        result = sock.connect_ex(('127.0.0.1', 8090))
        sock.close()
        if result == 0:
            print("⚠ 网关在运行但没有 PID 文件，请手动停止")
            return False
    except Exception:
        pass

    print("✓ 网关未运行")
    return True


def is_gateway_running() -> bool:
    """检查网关是否正在运行"""
    import socket

    # 方法 1: 检查 PID 文件
    pid_dir = Path.home() / ".cache" / "pi-llm-server" / "pids"
    pid_file = pid_dir / "gateway.pid"

    if pid_file.exists():
        try:
            pid = int(pid_file.read_text().strip())
            os.kill(pid, 0)
            return True
        except (ValueError, OSError):
            pid_file.unlink(missing_ok=True)

    # 方法 2: 检查端口
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        result = sock.connect_ex(('127.0.0.1', 8090))
        sock.close()
        return result == 0
    except Exception:
        return False


def show_full_status():
    """显示网关 + 后台服务状态"""
    print("=" * 60)
    print("PI-LLM-Server 全部服务状态")
    print("=" * 60)
    print()

    # 网关状态
    gateway_status = "运行中" if is_gateway_running() else "已停止"
    gateway_symbol = "✓" if is_gateway_running() else "✗"
    print(f"  {gateway_symbol} {'统一网关':20s} {gateway_status:10s} 端口：8090")

    # 后台服务状态
    from pi_llm_server.launcher.service_manager import SERVICE_CONFIG, is_service_running, get_service_pid

    for name, cfg in SERVICE_CONFIG.items():
        status = "运行中" if is_service_running(name) else "已停止"
        pid = get_service_pid(name)
        pid_str = f"(PID: {pid})" if pid else ""
        symbol = "✓" if is_service_running(name) else "✗"
        print(f"  {symbol} {cfg['name']:20s} {status:10s} 端口：{cfg['port']:5d} {pid_str}")

    print()
    print("=" * 60)


def start_all_services():
    """一站式启动：后台服务 + 网关"""
    print("=" * 60)
    print("启动所有服务（后台服务 + 网关）")
    print("=" * 60)
    print()

    # 确保配置文件存在
    config_file = ensure_config_exists()

    # 启动后台服务
    from pi_llm_server.launcher.service_manager import start_all
    start_all()

    print()
    print("-" * 60)
    print()

    # 启动网关
    start_gateway_background(config_file=config_file)

    print()
    print("=" * 60)
    print("所有服务已启动")
    print("=" * 60)


def stop_all_services():
    """一站式停止：网关 + 后台服务"""
    print("=" * 60)
    print("停止所有服务")
    print("=" * 60)
    print()

    # 先停止网关
    stop_gateway()

    print()

    # 停止后台服务
    from pi_llm_server.launcher.service_manager import stop_all
    stop_all()

    print()
    print("=" * 60)
    print("所有服务已停止")
    print("=" * 60)


def run_gateway():
    """运行网关服务（前台模式）"""
    parser = argparse.ArgumentParser(
        description="PI-LLM Server - 统一 LLM 服务网关",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
示例用法:
  # 使用默认配置文件（~/.config/pi-llm-server/config.yaml）
  pi-llm-server

  # 指定配置文件
  pi-llm-server --config /path/to/config.yaml

  # 指定端口和日志级别
  pi-llm-server --port 8090 --log-level debug

  # 使用 python -m 方式启动
  python -m pi_llm_server --port 8090
        """
    )

    parser.add_argument(
        "--config", "-c",
        default=None,
        help=f"配置文件路径（默认：{DEFAULT_CONFIG_FILE}）"
    )

    parser.add_argument(
        "--host",
        default=None,
        help="服务主机地址（默认：从配置文件读取）"
    )

    parser.add_argument(
        "--port", "-p",
        type=int,
        default=None,
        help="服务端口（默认：从配置文件读取）"
    )

    parser.add_argument(
        "--log-level",
        default=None,
        choices=["debug", "info", "warning", "error"],
        help="日志级别（默认：从配置文件读取）"
    )

    args = parser.parse_args()

    # 确定配置文件路径
    if args.config:
        config_path = Path(args.config)
    else:
        config_path = ensure_config_exists()

    # 检查配置文件是否存在
    if not config_path.exists():
        print(f"错误：配置文件不存在：{config_path}")
        print("请复制 config.example.yaml 并修改配置")
        sys.exit(1)

    # 初始化日志
    log_level = args.log_level or "info"
    log_dir = Path.home() / ".cache" / "pi-llm-server" / "logs"
    log_dir.mkdir(parents=True, exist_ok=True)
    init_default_logging("pi-llm-server", log_level, str(log_dir))

    import logging
    logger = logging.getLogger(__name__)

    # 加载配置
    try:
        config_manager = init_config(str(config_path))
        logger.info(f"配置文件加载成功：{config_path}")
    except Exception as e:
        logger.error(f"配置文件加载失败：{e}")
        sys.exit(1)

    # 命令行参数覆盖配置文件
    host = args.host or config_manager.config.server.host
    port = args.port or config_manager.config.server.port
    if args.log_level:
        config_manager.config.server.log_level = args.log_level

    # 初始化服务
    initialize_services(config_manager)

    # 启动服务
    logger.info(f"启动服务：http://{host}:{port}")

    # 当 workers > 1 时，uvicorn 要求传递 import string 而不是 app 对象
    # 但多 worker 模式下，每个 worker 会重新导入模块，无法共享 config_manager
    # 因此建议生产环境使用反向代理（如 nginx）实现多进程，而非 uvicorn workers
    workers = config_manager.config.server.workers
    if workers > 1:
        logger.warning(f"多 worker 模式 ({workers}) 下配置管理器无法在 worker 间共享，建议使用反向代理实现多进程。")
        logger.warning("临时使用单 worker 模式启动，如需多 worker 请使用 nginx 等反向代理")
        workers = 1  # 临时降级为单 worker

    # 单 worker 模式，直接传递 app 对象
    uvicorn.run(
        app,
        host=host,
        port=port,
        log_level=config_manager.config.server.log_level,
        workers=workers,
    )


def main():
    """主函数 - 支持一站式启动命令"""
    # 检查子命令
    if len(sys.argv) > 1:
        if sys.argv[1] == 'start-all':
            # 一站式启动：后台服务 + 网关
            start_all_services()
        elif sys.argv[1] == 'stop-all':
            # 一站式停止：网关 + 后台服务
            stop_all_services()
        elif sys.argv[1] == 'status':
            # 显示全部服务状态
            show_full_status()
        elif sys.argv[1] == 'services':
            # 传递给 service_manager（只管理后台服务）
            sys.argv.pop(0)  # 移除 'pi-llm-server' 或脚本名
            from pi_llm_server.launcher.service_manager import main as service_main
            service_main()
        elif sys.argv[1] in ('-h', '--help'):
            print("""
PI-LLM-Server - 统一 LLM 服务网关

用法:
  pi-llm-server [命令] [选项]

命令:
  start-all     启动所有服务（后台服务 + 网关）
  stop-all      停止所有服务（网关 + 后台服务）
  status        查看所有服务状态（网关 + 后台服务）
  services      后台服务管理（start/stop/restart/status）
  <无命令>      仅启动统一网关（默认行为）

示例:
  pi-llm-server start-all              # 一站式启动所有服务
  pi-llm-server stop-all               # 一站式停止所有服务
  pi-llm-server status                 # 查看所有服务状态
  pi-llm-server services start --all   # 仅启动后台服务
  pi-llm-server --port 8090            # 仅启动网关，指定端口
            """)
        else:
            # 默认启动网关
            run_gateway()
    else:
        # 默认启动网关
        run_gateway()


if __name__ == "__main__":
    main()



"""
允许通过 python -m pi_llm_server 启动服务

用法:
    # 仅启动统一网关（默认）
    python -m pi_llm_server

    # 启动所有服务（后台服务 + 网关）
    python -m pi_llm_server start-all

    # 停止所有服务
    python -m pi_llm_server stop-all

    # 查看所有服务状态
    python -m pi_llm_server status

    # 管理后台服务
    python -m pi_llm_server services start --all
    python -m pi_llm_server services status

    # 使用 CLI 参数启动网关
    python -m pi_llm_server --port 8090
"""
from pi_llm_server.cli import main

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Embedding 服务模块

提供文本向量化服务，包括:
- /v1/embeddings: 生成 embedding 向量
- /v1/similarity: 计算文本相似度
"""

from __future__ import annotations

import httpx
import logging
from typing import Union, List, Optional
from fastapi import APIRouter, HTTPException
from pydantic import BaseModel, Field

from ..config import ServiceConfig

logger = logging.getLogger(__name__)


# ============ 请求/响应模型 ============

class EmbeddingRequest(BaseModel):
    """Embedding 请求"""
    input: Union[str, List[str]] = Field(..., description="输入文本或文本列表")
    model: Optional[str] = Field(default=None, description="模型 ID")
    encoding_format: Optional[str] = Field(default="float", description="编码格式：float | base64")


class EmbeddingResponse(BaseModel):
    """Embedding 响应"""
    object: str = "list"
    data: List[dict]
    model: str
    usage: dict


class SimilarityRequest(BaseModel):
    """相似度计算请求"""
    text1: str = Field(..., description="第一个文本")
    text2: str = Field(..., description="第二个文本")


class SimilarityResponse(BaseModel):
    """相似度计算响应"""
    similarity: float


# ============ 服务类 ============

class EmbeddingService:
    """Embedding 服务 (Client + Router)"""

    def __init__(self, config: ServiceConfig):
        """
        初始化 Embedding 服务

        Args:
            config: 服务配置
        """
        self.config = config
        self.base_url = config.base_url
        self.timeout = config.timeout_seconds
        self.max_retries = config.max_retries

        # HTTP 客户端
        self.client = httpx.AsyncClient(
            base_url=self.base_url,
            timeout=httpx.Timeout(self.timeout),
            limits=httpx.Limits(max_keepalive_connections=10, max_connections=20)
        )

        # Router
        self.router = APIRouter(tags=["embedding"])
        self._register_routes()

        # 服务信息
        self.models = config.models if hasattr(config, 'models') else []

    async def create_embeddings(
        self,
        input_text: Union[str, List[str]],
        model: Optional[str] = None,
        encoding_format: Optional[str] = "float",
    ) -> dict:
        """
        调用 embedding 服务生成向量

        Args:
            input_text: 输入文本或文本列表
            model: 模型 ID (可选)
            encoding_format: 编码格式：float | base64 (可选，默认 float)

        Returns:
            dict: embedding 响应

        Raises:
            HTTPException: 请求失败时抛出
        """
        request_data = {"input": input_text}
        if model:
            request_data["model"] = model
        if encoding_format:
            request_data["encoding_format"] = encoding_format

        for attempt in range(self.max_retries):
            try:
                response = await self.client.post(
                    "/v1/embeddings",
                    json=request_data,
                )
                response.raise_for_status()
                return response.json()
            except httpx.TimeoutException as e:
                logger.warning(f"Embedding 请求超时 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=504, detail="Embedding 服务响应超时")
            except httpx.HTTPError as e:
                logger.error(f"Embedding 请求失败 (attempt {attempt + 1}/{self.max_retries}): {e}")
                if attempt == self.max_retries - 1:
                    raise HTTPException(status_code=502, detail=f"Embedding 服务错误：{str(e)}")

        return None

    async def calculate_similarity(self, text1: str, text2: str) -> float:
        """
        计算文本相似度

        Args:
            text1: 第一个文本
            text2: 第二个文本

        Returns:
            float: 相似度分数 (0-1)

        Raises:
            HTTPException: 请求失败时抛出
        """
        try:
            response = await self.client.post(
                "/v1/similarity",
                json={"text1": text1, "text2": text2},
            )
            response.raise_for_status()
            result = response.json()
            return result.get("similarity", 0.0)
        except httpx.TimeoutException as e:
            logger.warning(f"相似度计算超时：{e}")
            raise HTTPException(status_code=504, detail="相似度计算超时")
        except httpx.HTTPError as e:
            logger.error(f"相似度计算失败：{e}")
            raise HTTPException(status_code=502, detail=f"相似度计算失败：{str(e)}")

    async def health_check(self) -> dict:
        """
        健康检查

        Returns:
            dict: 健康状态
        """
        try:
            response = await self.client.get("/health", timeout=5.0)
            if response.status_code == 200:
                return {"status": "healthy", "latency_ms": response.elapsed.total_seconds() * 1000}
            else:
                return {"status": "unhealthy", "status_code": response.status_code}
        except Exception as e:
            return {"status": "unhealthy", "error": str(e)}

    async def get_models(self) -> List[dict]:
        """
        获取可用模型列表

        Returns:
            List[dict]: 模型列表
        """
        try:
            response = await self.client.get("/v1/models", timeout=5.0)
            if response.status_code == 200:
                data = response.json()
                # 添加服务信息
                models = data.get("data", [])
                for model in models:
                    model["service"] = "embedding"
                return models
            return []
        except Exception as e:
            logger.warning(f"获取模型列表失败：{e}")
            return []

    def _register_routes(self):
        """注册 FastAPI 路由"""

        @self.router.post("/v1/embeddings", tags=["embedding"])
        async def embeddings_endpoint(request: EmbeddingRequest):
            """生成 embedding 向量"""
            return await self.create_embeddings(request.input, request.model)

        @self.router.post("/v1/similarity", tags=["embedding"])
        async def similarity_endpoint(request: SimilarityRequest):
            """计算两个文本的余弦相似度"""
            score = await self.calculate_similarity(request.text1, request.text2)
            return {"similarity": score}

    async def close(self):
        """关闭 HTTP 客户端"""
        await self.client.aclose()


# ============ 全局实例 ============

embedding_service: Optional[EmbeddingService] = None


def init_embedding_service(config: ServiceConfig) -> EmbeddingService:
    """
    初始化 Embedding 服务

    Args:
        config: 服务配置

    Returns:
        EmbeddingService: 服务实例
    """
    global embedding_service
    embedding_service = EmbeddingService(config)
    logger.info(f"Embedding 服务初始化完成：{config.base_url}")
    return embedding_service


def get_embedding_service() -> EmbeddingService:
    """获取 Embedding 服务实例"""
    if not embedding_service:
        raise RuntimeError("Embedding 服务未初始化")
    return embedding_service
